# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os
import tempfile

import onnx
import pytest
import torch

from otx.algorithms.common.adapters.mmdeploy.apis import NaiveExporter
from otx.algorithms.common.adapters.mmdeploy.utils.onnx import (
    prepare_onnx_for_openvino,
    remove_nodes_by_op_type,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.algorithms.common.adapters.mmdeploy.test_helpers import create_model


@e2e_pytest_unit
def test_remove_nodes_by_op_type():
    model = create_model("mmcls")

    with tempfile.TemporaryDirectory() as tempdir:
        onnx_path = NaiveExporter.torch2onnx(
            tempdir,
            model,
            {"img": [torch.zeros((1, 50, 50, 3))], "img_metas": []},
        )
        assert os.path.exists(onnx_path)

        onnx_model = onnx.load(onnx_path)
        onnx_model = remove_nodes_by_op_type(onnx_model, "Gemm")
        nodes = []
        for node in onnx_model.graph.node:
            if node.op_type == "Gemm":
                nodes.append(node)
        assert not nodes

        # NOTE: Currently does not work for multiple op_types
        with pytest.raises(AssertionError) as e:
            onnx_model = onnx.load(onnx_path)
            onnx_model = remove_nodes_by_op_type(onnx_model, "Conv")
            assert e.type == Exception, f"{e}"


@e2e_pytest_unit
def test_prepare_onnx_for_openvino():

    model = create_model("mmcls")

    with tempfile.TemporaryDirectory() as tempdir:
        onnx_path = NaiveExporter.torch2onnx(
            tempdir,
            model,
            {"img": [torch.zeros((1, 50, 50, 3))], "img_metas": []},
        )
        assert os.path.exists(onnx_path)

        prepare_onnx_for_openvino(onnx_path, onnx_path)
        onnx_model = onnx.load(onnx_path)
        nodes = []
        for node in onnx_model.graph.node:
            if node.op_type == "Mark":
                nodes.append(node)
        assert not nodes



#
# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#


import json
from random import randint

import pytest

from otx.api.entities.id import ID
from otx.api.entities.label import Color, Domain, LabelEntity
from otx.api.entities.label_schema import (
    LabelGraph,
    LabelGroup,
    LabelGroupType,
    LabelSchemaEntity,
    LabelTree,
)
from otx.api.serialization.datetime_mapper import DatetimeMapper
from otx.api.serialization.label_mapper import (
    ColorMapper,
    LabelGraphMapper,
    LabelGroupMapper,
    LabelMapper,
    LabelSchemaMapper,
    label_schema_to_bytes,
)
from otx.api.utils.time_utils import now
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestColorMapper:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_color_serialization(self):
        """
        This test serializes Color and checks serialized representation.
        Then it compares deserialized Color with original one.
        """

        red = randint(0, 255)  # nosec
        green = randint(0, 255)  # nosec
        blue = randint(0, 255)  # nosec
        alpha = randint(0, 255)  # nosec
        color = Color(red, green, blue, alpha)
        serialized = ColorMapper.forward(color)
        assert serialized == {"red": red, "green": green, "blue": blue, "alpha": alpha}

        deserialized = ColorMapper.backward(serialized)
        assert color == deserialized


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLabelEntityMapper:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_label_entity_serialization(self):
        """
        This test serializes LabelEntity and checks serialized representation.
        Then it compares deserialized LabelEntity with original one.
        """

        cur_date = now()
        red = randint(0, 255)  # nosec
        green = randint(0, 255)  # nosec
        blue = randint(0, 255)  # nosec
        alpha = randint(0, 255)  # nosec

        label = LabelEntity(
            name="my_label",
            domain=Domain.DETECTION,
            color=Color(red, green, blue, alpha),
            hotkey="ctrl+1",
            creation_date=cur_date,
            is_empty=False,
            id=ID("0000213"),
        )
        serialized = LabelMapper.forward(label)

        assert serialized == {
            "_id": "0000213",
            "name": "my_label",
            "color": {"red": red, "green": green, "blue": blue, "alpha": alpha},
            "hotkey": "ctrl+1",
            "domain": "DETECTION",
            "creation_date": DatetimeMapper.forward(cur_date),
            "is_empty": False,
            "is_anomalous": False,
        }

        deserialized = LabelMapper.backward(serialized)
        assert label == deserialized


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLabelSchemaEntityMapper:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_flat_label_schema_serialization(self):
        """
        This test serializes flat LabelSchema and checks serialized representation.
        Then it compares deserialized LabelSchema with original one.
        """

        cur_date = now()
        names = ["cat", "dog", "mouse"]
        colors = [
            Color(
                randint(0, 255),  # nosec
                randint(0, 255),  # nosec
                randint(0, 255),  # nosec
                randint(0, 255),  # nosec
            )  # nosec  # noqa
            for _ in range(3)
        ]
        labels = [
            LabelEntity(
                name=name,
                domain=Domain.CLASSIFICATION,
                creation_date=cur_date,
                id=ID(i),
                color=colors[i],
            )
            for i, name in enumerate(names)
        ]
        label_schema = LabelSchemaEntity.from_labels(labels)
        serialized = LabelSchemaMapper.forward(label_schema)

        assert serialized == {
            "label_tree": {"type": "tree", "directed": True, "nodes": [], "edges": []},
            "label_groups": [
                {
                    "_id": label_schema.get_groups()[0].id_,
                    "name": "from_label_list",
                    "label_ids": ["0", "1", "2"],
                    "relation_type": "EXCLUSIVE",
                }
            ],
            "all_labels": {
                "0": {
                    "_id": "0",
                    "name": "cat",
                    "color": ColorMapper.forward(colors[0]),
                    "hotkey": "",
                    "domain": "CLASSIFICATION",
                    "creation_date": DatetimeMapper.forward(cur_date),
                    "is_empty": False,
                    "is_anomalous": False,
                },
                "1": {
                    "_id": "1",
                    "name": "dog",
                    "color": ColorMapper.forward(colors[1]),
                    "hotkey": "",
                    "domain": "CLASSIFICATION",
                    "creation_date": DatetimeMapper.forward(cur_date),
                    "is_empty": False,
                    "is_anomalous": False,
                },
                "2": {
                    "_id": "2",
                    "name": "mouse",
                    "color": ColorMapper.forward(colors[2]),
                    "hotkey": "",
                    "domain": "CLASSIFICATION",
                    "creation_date": DatetimeMapper.forward(cur_date),
                    "is_empty": False,
                    "is_anomalous": False,
                },
            },
        }

        deserialized = LabelSchemaMapper.backward(serialized)
        assert label_schema == deserialized

        # Checking value returned by "label_schema_to_bytes" function
        expected_label_schema_to_bytes = json.dumps(serialized, indent=4).encode()
        actual_label_schema_to_bytes = label_schema_to_bytes(label_schema)
        assert actual_label_schema_to_bytes == expected_label_schema_to_bytes


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLabelGroupMapper:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_label_group_serialization(self):
        """
        This test serializes flat LabelGroup and checks serialized representation.
        Then it compares deserialized LabelGroup with original one.
        """

        names = ["cat", "dog", "mouse"]
        labels = [
            LabelEntity(
                name=name,
                domain=Domain.CLASSIFICATION,
                id=ID(str(i)),
            )
            for i, name in enumerate(names)
        ]
        label_group = LabelGroup(name="Test LabelGroup", labels=labels, group_type=LabelGroupType.EMPTY_LABEL)
        serialized = LabelGroupMapper.forward(label_group)
        assert serialized == {
            "_id": label_group.id_,
            "name": "Test LabelGroup",
            "label_ids": ["0", "1", "2"],
            "relation_type": "EMPTY_LABEL",
        }
        all_labels = {ID(str(i)): labels[i] for i in range(3)}

        deserialized = LabelGroupMapper.backward(instance=serialized, all_labels=all_labels)
        assert deserialized == label_group


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLabelGraphMapper:
    label_0 = LabelEntity(name="label_0", domain=Domain.SEGMENTATION, id=ID("0"))
    label_0_1 = LabelEntity(name="label_0_1", domain=Domain.SEGMENTATION, id=ID("0_1"))
    label_0_2 = LabelEntity(name="label_0_2", domain=Domain.SEGMENTATION, id=ID("0_2"))
    label_0_1_1 = LabelEntity(name="label_0_1_1", domain=Domain.SEGMENTATION, id=ID("0_1_1"))
    label_0_1_2 = LabelEntity(name="label_0_1_2", domain=Domain.SEGMENTATION, id=ID("0_1_2"))
    label_0_2_1 = LabelEntity(name="label_0_2_1", domain=Domain.SEGMENTATION, id=ID("0_2_1"))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_label_graph_forward(self):
        """
        <b>Description:</b>
        Check "LabelGraphMapper" class "forward" method

        <b>Input data:</b>
        "LabelGraph" and "LabelTree" objects

        <b>Expected results:</b>
        Test passes if dictionary returned by "forward" method is equal to expected

        <b>Steps</b>
        1. Check dictionary returned by "forward" method for "LabelGraph" object
        2. Check dictionary returned by "forward" method for "LabelTree" object
        """
        # Checking dictionary returned by "forward" for "LabelGraph"
        label_graph = LabelGraph(directed=False)
        label_graph.add_edges(
            [
                (self.label_0, self.label_0_1),
                (self.label_0, self.label_0_2),
                (self.label_0_1, self.label_0_1_1),
                (self.label_0_1, self.label_0_1_2),
                (self.label_0_1_1, self.label_0_1_2),
            ]
        )
        forward = LabelGraphMapper.forward(label_graph)
        assert forward == {
            "type": "graph",
            "directed": False,
            "nodes": ["0", "0_1", "0_2", "0_1_1", "0_1_2"],
            "edges": [
                ("0", "0_1"),
                ("0", "0_2"),
                ("0_1", "0_1_1"),
                ("0_1", "0_1_2"),
                ("0_1_1", "0_1_2"),
            ],
        }
        # Checking dictionary returned by "forward" for "LabelTree"
        label_tree = LabelTree()
        for parent, child in [
            (self.label_0, self.label_0_1),
            (self.label_0, self.label_0_2),
            (self.label_0_1, self.label_0_1_1),
            (self.label_0_1, self.label_0_1_2),
            (self.label_0_2, self.label_0_2_1),
        ]:
            label_tree.add_child(parent, child)
        forward = LabelGraphMapper.forward(label_tree)
        assert forward == {
            "type": "tree",
            "directed": True,
            "nodes": ["0_1", "0", "0_2", "0_1_1", "0_1_2", "0_2_1"],
            "edges": [
                ("0_1", "0"),
                ("0_2", "0"),
                ("0_1_1", "0_1"),
                ("0_1_2", "0_1"),
                ("0_2_1", "0_2"),
            ],
        }

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_label_graph_backward(self):
        """
        <b>Description:</b>
        Check "LabelGraphMapper" class "backward" method

        <b>Input data:</b>
        Dictionary object to deserialize, labels list

        <b>Expected results:</b>
        Test passes if "LabelGraph" or "LabelTree" object returned by "backward" method is equal to expected

        <b>Steps</b>
        1. Check dictionary returned by "backward" method for "LabelGraph" object
        2. Check dictionary returned by "backward" method for "LabelTree" object
        3. Check that "ValueError" exception is raised when unsupported type is specified as "type" key in dictionary
        object of "instance" parameter for "backward" method
        """
        # Checking dictionary returned by "backward" for "LabelGraph"
        forward = {
            "type": "graph",
            "directed": False,
            "nodes": ["0", "0_1", "0_2", "0_1_1"],
            "edges": [("0", "0_1"), ("0", "0_2"), ("0_1", "0_1_1"), ("0_1_1", "0_2")],
        }
        labels = {
            ID("0"): self.label_0,
            ID("0_1"): self.label_0_1,
            ID("0_2"): self.label_0_2,
            ID("0_1_1"): self.label_0_1_1,
        }
        expected_backward = LabelGraph(directed=False)
        expected_backward.add_edges(
            [
                (self.label_0, self.label_0_1),
                (self.label_0, self.label_0_2),
                (self.label_0_1, self.label_0_1_1),
                (self.label_0_1_1, self.label_0_2),
            ]
        )
        actual_backward = LabelGraphMapper.backward(instance=forward, all_labels=labels)
        assert actual_backward == expected_backward
        # Checking dictionary returned by "backward" for "LabelTree"
        forward = {
            "type": "tree",
            "directed": True,
            "nodes": ["0_1", "0", "0_2", "0_1_1", "0_2_1"],
            "edges": [("0_1", "0"), ("0_2", "0"), ("0_1_1", "0_1"), ("0_2_1", "0_2")],
        }
        labels = {
            ID("0"): self.label_0,
            ID("0_1"): self.label_0_1,
            ID("0_2"): self.label_0_2,
            ID("0_1_1"): self.label_0_1_1,
            ID("0_1_2"): self.label_0_1_2,
            ID("0_2_1"): self.label_0_2_1,
        }
        expected_backward = LabelTree()
        for parent, child in [
            (self.label_0, self.label_0_1),
            (self.label_0, self.label_0_2),
            (self.label_0_1, self.label_0_1_1),
            (self.label_0_2, self.label_0_2_1),
        ]:
            expected_backward.add_child(parent, child)
        actual_backward = LabelGraphMapper.backward(instance=forward, all_labels=labels)
        assert actual_backward == expected_backward
        # Checking "ValueError" exception raised when unsupported type specified as "type" in dictionary "instance" for
        # "backward"
        forward = {
            "type": "rectangle",
            "directed": True,
            "nodes": ["0_1", "0", "0_2", "0_1_1", "0_2_1"],
            "edges": [("0_1", "0"), ("0_2", "0"), ("0_1_1", "0_1"), ("0_2_1", "0_2")],
        }
        with pytest.raises(ValueError):
            LabelGraphMapper.backward(instance=forward, all_labels=labels)



# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os.path as osp
import random
import time
from typing import Optional

import cv2 as cv
import numpy as np
import pytest
from bson import ObjectId

from otx.algorithms.classification.adapters.mmcls.task import MMClassificationTask
from otx.algorithms.common.tasks.base_task import OTXTask
from otx.api.configuration.helper import create
from otx.api.entities.annotation import (
    Annotation,
    AnnotationSceneEntity,
    AnnotationSceneKind,
)
from otx.api.entities.dataset_item import DatasetItemEntity
from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.id import ID
from otx.api.entities.image import Image
from otx.api.entities.inference_parameters import InferenceParameters
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.label_schema import LabelGroup, LabelGroupType, LabelSchemaEntity
from otx.api.entities.metrics import Performance
from otx.api.entities.model import ModelEntity
from otx.api.entities.model_template import parse_model_template
from otx.api.entities.resultset import ResultSetEntity
from otx.api.entities.scored_label import ScoredLabel
from otx.api.entities.shapes.rectangle import Rectangle
from otx.api.entities.subset import Subset
from otx.api.entities.task_environment import TaskEnvironment
from otx.api.entities.train_parameters import TrainParameters
from otx.api.usecases.tasks.interfaces.export_interface import ExportType
from tests.test_suite.e2e_test_system import e2e_pytest_api

DEFAULT_CLS_TEMPLATE_DIR = osp.join("otx/algorithms/classification", "configs", "efficientnet_b0_cls_incr")


def task_eval(task: OTXTask, model: ModelEntity, dataset: DatasetEntity) -> Performance:
    start_time = time.time()
    result_dataset = task.infer(dataset.with_empty_annotations())
    end_time = time.time()
    print(f"{len(dataset)} analysed in {end_time - start_time} seconds")
    result_set = ResultSetEntity(model=model, ground_truth_dataset=dataset, prediction_dataset=result_dataset)
    task.evaluate(result_set)
    assert result_set.performance is not None
    return result_set.performance


class ClassificationTaskAPIBase:
    @staticmethod
    def generate_label_schema(not_empty_labels, multilabel=False, hierarchical=False):
        assert len(not_empty_labels) > 1

        label_schema = LabelSchemaEntity()
        if multilabel:
            emptylabel = LabelEntity(name="Empty label", is_empty=True, domain=Domain.CLASSIFICATION)
            empty_group = LabelGroup(name="empty", labels=[emptylabel], group_type=LabelGroupType.EMPTY_LABEL)
            for label in not_empty_labels:
                label_schema.add_group(
                    LabelGroup(
                        name=label.name,
                        labels=[label],
                        group_type=LabelGroupType.EXCLUSIVE,
                    )
                )
            label_schema.add_group(empty_group)
        elif hierarchical:
            single_label_classes = ["b", "g", "r"]
            multi_label_classes = ["w", "p"]
            emptylabel = LabelEntity(name="Empty label", is_empty=True, domain=Domain.CLASSIFICATION)
            empty_group = LabelGroup(name="empty", labels=[emptylabel], group_type=LabelGroupType.EMPTY_LABEL)
            single_labels = []
            for label in not_empty_labels:
                if label.name in multi_label_classes:
                    label_schema.add_group(
                        LabelGroup(
                            name=label.name,
                            labels=[label],
                            group_type=LabelGroupType.EXCLUSIVE,
                        )
                    )
                    if empty_group not in label_schema.get_groups(include_empty=True):
                        label_schema.add_group(empty_group)
                elif label.name in single_label_classes:
                    single_labels.append(label)
            if single_labels:
                single_label_group = LabelGroup(
                    name="labels",
                    labels=single_labels,
                    group_type=LabelGroupType.EXCLUSIVE,
                )
                label_schema.add_group(single_label_group)
        else:
            main_group = LabelGroup(
                name="labels",
                labels=not_empty_labels,
                group_type=LabelGroupType.EXCLUSIVE,
            )
            label_schema.add_group(main_group)
        return label_schema

    @staticmethod
    def setup_configurable_parameters(template_dir, num_iters=10):
        model_template = parse_model_template(osp.join(template_dir, "template.yaml"))
        hyper_parameters = create(model_template.hyper_parameters.data)
        hyper_parameters.learning_parameters.num_iters = num_iters
        return hyper_parameters, model_template

    @staticmethod
    def init_environment(params, model_template, multilabel, hierarchical, number_of_images=10):
        resolution = (224, 224)
        if hierarchical:
            colors = [(0, 255, 0), (0, 0, 255), (255, 0, 0), (0, 0, 0), (230, 230, 250)]
            cls_names = ["b", "g", "r", "w", "p"]
            texts = ["Blue", "Green", "Red", "White", "Purple"]
        else:
            colors = [(0, 255, 0), (0, 0, 255)]
            cls_names = ["b", "g"]
            texts = ["Blue", "Green"]
        env_labels = [
            LabelEntity(name=name, domain=Domain.CLASSIFICATION, is_empty=False, id=ID(i))
            for i, name in enumerate(cls_names)
        ]

        items = []

        for _ in range(0, number_of_images):
            for j, lbl in enumerate(env_labels):
                class_img = np.zeros((*resolution, 3), dtype=np.uint8)
                class_img[:] = colors[j]
                class_img = cv.putText(
                    class_img,
                    texts[j],
                    (50, 50),
                    cv.FONT_HERSHEY_SIMPLEX,
                    0.8 + j * 0.2,
                    colors[j - 1],
                    2,
                    cv.LINE_AA,
                )

                image = Image(data=class_img)
                labels = [ScoredLabel(label=lbl, probability=1.0)]
                shapes = [Annotation(Rectangle.generate_full_box(), labels)]
                annotation_scene = AnnotationSceneEntity(kind=AnnotationSceneKind.ANNOTATION, annotations=shapes)
                items.append(DatasetItemEntity(media=image, annotation_scene=annotation_scene))

        rng = random.Random()
        rng.seed(100)
        rng.shuffle(items)
        for i, _ in enumerate(items):
            subset_region = i / number_of_images
            if subset_region >= 0.9:
                subset = Subset.TESTING
            elif subset_region >= 0.6:
                subset = Subset.VALIDATION
            else:
                subset = Subset.TRAINING
            items[i].subset = subset

        dataset = DatasetEntity(items)
        labels_schema = ClassificationTaskAPIBase.generate_label_schema(
            dataset.get_labels(), multilabel=multilabel, hierarchical=hierarchical
        )
        environment = TaskEnvironment(
            model=None,
            hyper_parameters=params,
            label_schema=labels_schema,
            model_template=model_template,
        )
        return environment, dataset


class TestClassificationTaskAPI(ClassificationTaskAPIBase):
    @e2e_pytest_api
    def test_reading_classification_cls_incr_model_template(self):
        classification_template = [
            "efficientnet_b0_cls_incr",
            "efficientnet_v2_s_cls_incr",
            "mobilenet_v3_large_1_cls_incr",
        ]
        for model_template in classification_template:
            parse_model_template(osp.join("otx/algorithms/classification", "configs", model_template, "template.yaml"))

    @e2e_pytest_api
    @pytest.mark.parametrize(
        "multilabel,hierarchical",
        [(False, False), (True, False), (False, True)],
        ids=["multiclass", "multilabel", "hierarchical"],
    )
    def test_training_progress_tracking(self, multilabel, hierarchical):
        hyper_parameters, model_template = self.setup_configurable_parameters(DEFAULT_CLS_TEMPLATE_DIR, num_iters=10)
        task_environment, dataset = self.init_environment(
            hyper_parameters, model_template, multilabel, hierarchical, 20
        )
        task = MMClassificationTask(task_environment=task_environment)
        print("Task initialized, model training starts.")

        training_progress_curve = []

        def progress_callback(progress: float, score: Optional[float] = None):
            training_progress_curve.append(progress)

        train_parameters = TrainParameters()
        train_parameters.update_progress = progress_callback
        output_model = ModelEntity(
            dataset,
            task_environment.get_model_configuration(),
        )
        task.train(dataset, output_model, train_parameters)

        assert len(training_progress_curve) > 0
        assert np.all(training_progress_curve[1:] >= training_progress_curve[:-1])

    @e2e_pytest_api
    @pytest.mark.parametrize(
        "multilabel,hierarchical",
        [(False, False), (True, False), (False, True)],
        ids=["multiclass", "multilabel", "hierarchical"],
    )
    def test_inference_progress_tracking(self, multilabel, hierarchical):
        hyper_parameters, model_template = self.setup_configurable_parameters(DEFAULT_CLS_TEMPLATE_DIR, num_iters=5)
        task_environment, dataset = self.init_environment(
            hyper_parameters, model_template, multilabel, hierarchical, 20
        )
        task = MMClassificationTask(task_environment=task_environment)
        print("Task initialized, model inference starts.")

        inference_progress_curve = []

        def progress_callback(progress: int):
            inference_progress_curve.append(progress)

        inference_parameters = InferenceParameters()
        inference_parameters.update_progress = progress_callback
        task.infer(dataset.with_empty_annotations(), inference_parameters)

        assert len(inference_progress_curve) > 0
        assert np.all(inference_progress_curve[1:] >= inference_progress_curve[:-1])

    @e2e_pytest_api
    @pytest.mark.parametrize(
        "multilabel,hierarchical",
        [(False, False), (True, False), (False, True)],
        ids=["multiclass", "multilabel", "hierarchical"],
    )
    def test_inference_task(self, multilabel, hierarchical):
        # Prepare pretrained weights
        hyper_parameters, model_template = self.setup_configurable_parameters(DEFAULT_CLS_TEMPLATE_DIR, num_iters=2)
        classification_environment, dataset = self.init_environment(
            hyper_parameters, model_template, multilabel, hierarchical, 50
        )
        val_dataset = dataset.get_subset(Subset.VALIDATION)

        train_task = MMClassificationTask(task_environment=classification_environment)

        training_progress_curve = []

        def progress_callback(progress: float, score: Optional[float] = None):
            training_progress_curve.append(progress)

        train_parameters = TrainParameters()
        train_parameters.update_progress = progress_callback
        trained_model = ModelEntity(
            dataset,
            classification_environment.get_model_configuration(),
        )
        train_task.train(dataset, trained_model, train_parameters)
        performance_after_train = task_eval(train_task, trained_model, val_dataset)

        # Create InferenceTask
        classification_environment.model = trained_model
        inference_task = MMClassificationTask(task_environment=classification_environment)

        performance_after_load = task_eval(inference_task, trained_model, val_dataset)

        assert performance_after_train == performance_after_load

        # Export
        exported_model = ModelEntity(
            dataset,
            classification_environment.get_model_configuration(),
            _id=ObjectId(),
        )
        inference_task.export(ExportType.OPENVINO, exported_model)



import argparse
from unittest.mock import patch

from otx.cli.tools import find as target_package
from tests.test_suite.e2e_test_system import e2e_pytest_unit


def create_mock_args(task=None, template=None, backbone=None):
    mock_args = argparse.Namespace()
    mock_args.task = task
    mock_args.template = template
    mock_args.backbone = backbone
    return mock_args


@e2e_pytest_unit
def test_generate_backbone_rows():
    backbone_meta = {
        "mock_backbone": {
            "required": ["first", "second"],
            "options": {"first": ["option1", "option2"], "second": ["option1", "option2"]},
            "available": True,
        }
    }
    rows = target_package.generate_backbone_rows(1, "mock_backbone", backbone_meta["mock_backbone"])

    assert rows == [
        ["1", "mock_backbone", "first", "option1, option2"],
        ["", "", "second", "option1, option2"],
    ]


@e2e_pytest_unit
def test_main():
    mock_args = create_mock_args()
    with patch("argparse.ArgumentParser.parse_args", return_value=mock_args):
        result = target_package.main()

    assert result == {"retcode": 0, "task_type": None}


@e2e_pytest_unit
def test_main_with_template():
    mock_args = create_mock_args(template=True)
    with patch("argparse.ArgumentParser.parse_args", return_value=mock_args):
        result = target_package.main()

    assert result == {"retcode": 0, "task_type": None}


@e2e_pytest_unit
def test_main_with_task():
    mock_args = create_mock_args(task="CLASSIFICATION")
    with patch("argparse.ArgumentParser.parse_args", return_value=mock_args):
        with patch.object(target_package.Registry, "filter") as mock_filter:
            with patch("prettytable.PrettyTable.add_row"):
                result = target_package.main()

    assert result == {"retcode": 0, "task_type": "CLASSIFICATION"}
    mock_filter.assert_called_once_with(task_type="CLASSIFICATION")


@e2e_pytest_unit
def test_main_with_backbone():
    mock_args = create_mock_args(backbone=["backbone1"])
    with patch("argparse.ArgumentParser.parse_args", return_value=mock_args):
        with patch.object(target_package.Registry, "get_backbones") as mock_get_backbones:
            with patch("prettytable.PrettyTable.add_rows"):
                result = target_package.main()

    assert result == {"retcode": 0, "task_type": None}
    mock_get_backbones.assert_called_once_with(["backbone1"])



"""Initial file for mmdetection backbones."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from . import imgclsmob
from .mmov_backbone import MMOVBackbone

__all__ = ["imgclsmob", "MMOVBackbone"]



"""Unit Test for otx.algorithms.action.adapters.mmaction.models.recognizers.movinet_recognizer."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
from copy import deepcopy

import pytest
import torch
from mmaction.models.recognizers.recognizer3d import Recognizer3D

from otx.algorithms.action.adapters.mmaction.models.recognizers.movinet_recognizer import (
    MoViNetRecognizer,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockOTXMoViNet:
    pass


class MockModule:
    def __init__(self):
        self.backbone = MockOTXMoViNet()
        self._state_dict = {
            "classifier.0.conv_1.conv3d.weight": torch.rand(1, 1),
            "conv1.conv_1.conv3d.weight": torch.rand(1, 1),
        }
        self.is_export = False

    def state_dict(self):
        return self._state_dict


class TestMoViNetRecognizer:
    @pytest.fixture(autouse=True)
    def setup(self, mocker) -> None:
        mocker.patch.object(Recognizer3D, "__init__", return_value=None)
        MoViNetRecognizer._register_state_dict_hook = mocker.MagicMock()
        MoViNetRecognizer._register_load_state_dict_pre_hook = mocker.MagicMock()
        self.recognizer = MoViNetRecognizer()
        self.prefix = ""

    @e2e_pytest_unit
    def test_load_state_dict_pre_hook(self) -> None:
        """Test load_state_dict_pre_hook function."""
        module = MockModule()
        state_dict = module.state_dict()
        self.recognizer.load_state_dict_pre_hook(module, state_dict, prefix=self.prefix)

        for key in state_dict:
            if "classifier" in key:
                assert "cls_head.classifier.0.conv_1.conv3d.weight" in state_dict
            else:
                assert "backbone.conv1.conv_1.conv3d.weight" in state_dict

    @e2e_pytest_unit
    def test_state_dict_hook(self):
        """Test state_dict_hook function."""
        module = MockModule()
        state_dict = module.state_dict()
        state_dict_copy = deepcopy(state_dict)
        self.recognizer.load_state_dict_pre_hook(module, state_dict, prefix=self.prefix)
        # backward state dict
        self.recognizer.state_dict_hook(module, state_dict, prefix=self.prefix)

        assert state_dict.keys() == state_dict_copy.keys()



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from copy import deepcopy
from random import shuffle

import numpy as np
import openvino.runtime as ov
import pytest

from otx.core.ov.graph.graph import Graph, SortedDict
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestSortedDict:
    @e2e_pytest_unit
    def test(self):
        instance = SortedDict("key")
        orders = list("abcdefghijklmnopqrstuvwxyz")
        cands = list("abcdefghijklmnopqrstuvwxyz")
        shuffle(cands)

        for cand in cands:
            instance[cand] = {"edge": {"key": ord(cand)}}

        idx = 0
        for key in instance:
            assert key == orders[idx]
            idx += 1

        idx = len(orders) - 1
        for key in reversed(instance):
            assert key == orders[idx]
            idx -= 1

        repr(instance.keys())
        idx = 0
        for key in instance.keys():
            assert key == orders[idx]
            idx += 1

        idx = len(orders) - 1
        for key in reversed(instance.keys()):
            assert key == orders[idx]
            idx -= 1

        repr(instance.values())
        idx = 0
        for value in instance.values():
            assert value["edge"]["key"] == ord(orders[idx])
            idx += 1

        idx = len(orders) - 1
        for value in reversed(instance.values()):
            assert value["edge"]["key"] == ord(orders[idx])
            idx -= 1

        repr(instance.values())
        idx = 0
        for key, value in instance.items():
            assert key == orders[idx]
            assert value["edge"]["key"] == ord(orders[idx])
            idx += 1

        idx = len(orders) - 1
        for key, value in reversed(instance.items()):
            assert key == orders[idx]
            assert value["edge"]["key"] == ord(orders[idx])
            idx -= 1

        instance2 = deepcopy(instance)
        idx = 0
        for key, value in instance2.items():
            assert key == orders[idx]
            assert value["edge"]["key"] == ord(orders[idx])
            idx += 1

        instance.pop("i")
        assert "i" not in instance
        assert len(instance) == len(orders) - 1

        instance.clear()
        assert len(instance) == 0


class TestGraph:
    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        param = ov.opset10.parameter([1, 3, 64, 64], ov.Type.f32, name="in")
        constant = ov.opset10.constant(np.array([103.0, 116.0, 123.0]).reshape(1, 3, 1, 1), ov.Type.f32)
        node = ov.opset10.subtract(param, constant, "numpy")
        constant = ov.opset10.constant(np.random.normal(size=(32, 3, 3, 3)), ov.Type.f32)
        node = ov.opset10.convolution(node, constant, [2, 2], [1, 1], [1, 1], [1, 1], "explicit")
        constant = ov.opset10.constant(np.random.normal(size=(1, 32, 1, 1)), ov.Type.f32)
        node = ov.opset10.add(node, constant, "numpy")
        node = ov.opset10.clamp(node, 0, 6)
        result = ov.opset10.result(node, name="out")
        ov_model = ov.Model([result], [param], "model")

        self.graph = Graph.from_ov(ov_model)
        assert isinstance(self.graph, Graph)

    @e2e_pytest_unit
    def test_get_edge_data(self):
        nodes = [node for node in self.graph]
        assert self.graph.get_edge_data(nodes[0], nodes[-1]) is None
        assert self.graph.get_edge_data(nodes[0], nodes[2])

    @e2e_pytest_unit
    def test_remove_node(self):
        node = self.graph.get_nodes_by_types(["Subtract"])[0]
        predecessor = list(self.graph.predecessors(node))[0]
        successor = list(self.graph.successors(node))[0]
        self.graph.remove_node(node, keep_connect=True)
        assert self.graph.get_edge_data(predecessor, successor)

        node = self.graph.get_nodes_by_types(["Convolution"])[0]
        predecessor = list(self.graph.predecessors(node))[0]
        successor = list(self.graph.successors(node))[0]
        self.graph.remove_node(node, keep_connect=False)
        assert self.graph.get_edge_data(predecessor, successor) is None

    @e2e_pytest_unit
    def test_replace_node(self):
        node = self.graph.get_nodes_by_types(["Subtract"])[0]
        new_node = deepcopy(node)
        predecessors = list(self.graph.predecessors(node))
        successors = list(self.graph.successors(node))
        self.graph.replace_node(node, new_node)

        assert node not in self.graph
        assert new_node in self.graph
        assert predecessors == list(self.graph.predecessors(new_node))
        assert successors == list(self.graph.successors(new_node))

    @e2e_pytest_unit
    def test_add_edge(self):
        node = self.graph.get_nodes_by_types(["Subtract"])[0]
        new_node = deepcopy(node)
        predecessors = list(self.graph.predecessors(node))
        successors = list(self.graph.successors(node))
        self.graph.remove_node(node)

        for predecessor in predecessors:
            assert self.graph.get_edge_data(predecessor, new_node) is None
            self.graph.add_edge(predecessor, new_node)
            assert self.graph.get_edge_data(predecessor, new_node)

        for successor in successors:
            assert self.graph.get_edge_data(new_node, successor) is None
            self.graph.add_edge(new_node, successor)
            assert self.graph.get_edge_data(new_node, successor)

        assert new_node in self.graph

    @e2e_pytest_unit
    def test_get_nodes_by_type_pattern(self):
        node = self.graph.get_nodes_by_types(["Subtract"])[0]
        founds = self.graph.get_nodes_by_type_pattern(["Subtract", "Clamp"], node)
        for found in founds:
            start, end = found
            assert start == node
            assert start.type == "Subtract"
            assert end.type == "Clamp"

    @e2e_pytest_unit
    def test_remove_normalize_nodes(self):
        self.graph.remove_normalize_nodes()
        assert len(self.graph._normalize_nodes) == 0

    @e2e_pytest_unit
    def test_topological_sort(self):
        assert len(list(self.graph.topological_sort())) == len(self.graph)

    @e2e_pytest_unit
    def test_clean_up(self):
        nodes = self.graph.get_nodes_by_types(["Subtract"])
        self.graph.remove_node(nodes[0])
        n_nodes = len(self.graph)
        self.graph.clean_up()

        assert n_nodes > len(self.graph)



"""Supervised learning settings for video actor localization."""

# Copyright (C) 2022 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

# pylint: disable=invalid-name

optimizer = dict(type="SGD", lr=0.1, momentum=0.9, weight_decay=1e-5)
optimizer_config = dict(grad_clip=dict(max_norm=40, norm_type=2))

lr_config = dict(
    policy="CosineAnnealing",
    by_epoch=False,
    min_lr=0,
    warmup="linear",
    warmup_by_epoch=True,
    warmup_iters=2,
    warmup_ratio=0.1,
)
checkpoint_config = dict(interval=1)
workflow = [("train", 1)]
evaluation = dict(interval=1, save_best="mAP@0.5IOU", final_metric="mAP@0.5IOU")
log_config = dict(
    interval=10,
    hooks=[
        dict(type="TextLoggerHook"),
    ],
)
dist_params = dict(backend="nccl")
log_level = "INFO"
find_unused_parameters = False
# Temporary solution, gpu_ids is not used in otx
gpu_ids = [0]
seed = 2



import torch
from mmcv.utils import ConfigDict
from mmdet.models.builder import build_detector

from otx.algorithms.detection.adapters.mmdet.models.detectors.custom_two_stage_detector import (
    CustomTwoStageDetector,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestCustomTwoStageDetector:
    @e2e_pytest_unit
    def test_custom_two_stage_detector_build(self):
        model_cfg = ConfigDict(
            type="CustomTwoStageDetector",
            backbone=dict(
                type="ResNet",
                depth=50,
                num_stages=4,
                out_indices=(0, 1, 2, 3),
                frozen_stages=1,
                norm_cfg=dict(type="BN", requires_grad=True),
                norm_eval=True,
                style="pytorch",
            ),
            neck=dict(type="FPN", in_channels=[256, 512, 1024, 2048], out_channels=256, num_outs=5),
            rpn_head=dict(
                type="RPNHead",
                in_channels=256,
                feat_channels=256,
                anchor_generator=dict(
                    type="AnchorGenerator", scales=[8], ratios=[0.5, 1.0, 2.0], strides=[4, 8, 16, 32, 64]
                ),
                bbox_coder=dict(
                    type="DeltaXYWHBBoxCoder", target_means=[0.0, 0.0, 0.0, 0.0], target_stds=[1.0, 1.0, 1.0, 1.0]
                ),
                loss_cls=dict(type="CrossEntropyLoss", use_sigmoid=True, loss_weight=1.0),
                loss_bbox=dict(type="L1Loss", loss_weight=1.0),
            ),
            roi_head=dict(
                type="StandardRoIHead",
                bbox_roi_extractor=dict(
                    type="SingleRoIExtractor",
                    roi_layer=dict(type="RoIAlign", output_size=7, sampling_ratio=0.0),
                    out_channels=256,
                    featmap_strides=[4, 8, 16, 32],
                ),
                bbox_head=dict(
                    type="Shared2FCBBoxHead",
                    in_channels=256,
                    fc_out_channels=1024,
                    roi_feat_size=7,
                    num_classes=80,
                    bbox_coder=dict(
                        type="DeltaXYWHBBoxCoder", target_means=[0.0, 0.0, 0.0, 0.0], target_stds=[0.1, 0.1, 0.2, 0.2]
                    ),
                    reg_class_agnostic=False,
                    loss_cls=dict(type="CrossEntropyLoss", use_sigmoid=False, loss_weight=1.0),
                    loss_bbox=dict(type="L1Loss", loss_weight=1.0),
                ),
            ),
            # model training and testing settings
            train_cfg=dict(
                rpn=dict(
                    assigner=dict(
                        type="MaxIoUAssigner",
                        pos_iou_thr=0.7,
                        neg_iou_thr=0.3,
                        min_pos_iou=0.3,
                        match_low_quality=True,
                        ignore_iof_thr=-1,
                    ),
                    sampler=dict(
                        type="RandomSampler", num=256, pos_fraction=0.5, neg_pos_ub=-1, add_gt_as_proposals=False
                    ),
                    allowed_border=-1,
                    pos_weight=-1,
                    debug=False,
                ),
                rpn_proposal=dict(
                    nms_across_levels=False, nms_pre=2000, nms_post=1000, max_num=1000, nms_thr=0.7, min_bbox_size=0
                ),
                rcnn=dict(
                    assigner=dict(
                        type="MaxIoUAssigner",
                        pos_iou_thr=0.5,
                        neg_iou_thr=0.5,
                        min_pos_iou=0.5,
                        match_low_quality=False,
                        ignore_iof_thr=-1,
                    ),
                    sampler=dict(
                        type="RandomSampler", num=512, pos_fraction=0.25, neg_pos_ub=-1, add_gt_as_proposals=True
                    ),
                    pos_weight=-1,
                    debug=False,
                ),
            ),
            test_cfg=dict(
                rpn=dict(
                    nms_across_levels=False, nms_pre=1000, nms_post=1000, max_num=1000, nms_thr=0.7, min_bbox_size=0
                ),
                rcnn=dict(score_thr=0.05, nms=dict(type="nms", iou_threshold=0.5), max_per_img=100)
                # soft-nms is also supported for rcnn testing
                # e.g., nms=dict(type='soft_nms', iou_threshold=0.5, min_score=0.05)
            ),
            task_adapt=dict(
                src_classes=["person", "car"],
                dst_classes=["tree", "car", "person"],
            ),
        )

        model = build_detector(model_cfg)
        assert isinstance(model, CustomTwoStageDetector)

    @e2e_pytest_unit
    def test_custom_two_stage_detector_load_state_dict_pre_hook(self):
        chkpt_classes = ["person", "car"]
        model_classes = ["tree", "car", "person"]
        chkpt_dict = {
            "roi_head.bbox_head.fc_cls.weight": torch.tensor(
                [
                    [1, 1, 1, 1],
                    [2, 2, 2, 2],
                ]
            ),
            "roi_head.bbox_head.fc_cls.bias": torch.tensor(
                [
                    [1],
                    [2],
                ]
            ),
            "roi_head.bbox_head.fc_reg.weight": torch.tensor(
                [
                    [1, 1, 1, 1],
                    [1, 1, 1, 1],
                    [1, 1, 1, 1],
                    [1, 1, 1, 1],
                    [2, 2, 2, 2],
                    [2, 2, 2, 2],
                    [2, 2, 2, 2],
                    [2, 2, 2, 2],
                ]
            ),
            "roi_head.bbox_head.fc_reg.bias": torch.tensor(
                [
                    [1],
                    [1],
                    [1],
                    [1],
                    [2],
                    [2],
                    [2],
                    [2],
                ]
            ),
        }
        model_dict = {
            "roi_head.bbox_head.fc_cls.weight": torch.tensor(
                [
                    [3, 3, 3, 3],
                    [4, 4, 4, 4],
                    [5, 5, 5, 5],
                ]
            ),
            "roi_head.bbox_head.fc_cls.bias": torch.tensor(
                [
                    [3],
                    [4],
                    [5],
                ]
            ),
            "roi_head.bbox_head.fc_reg.weight": torch.tensor(
                [
                    [3, 3, 3, 3],
                    [3, 3, 3, 3],
                    [3, 3, 3, 3],
                    [3, 3, 3, 3],
                    [4, 4, 4, 4],
                    [4, 4, 4, 4],
                    [4, 4, 4, 4],
                    [4, 4, 4, 4],
                    [5, 5, 5, 5],
                    [5, 5, 5, 5],
                    [5, 5, 5, 5],
                    [5, 5, 5, 5],
                ]
            ),
            "roi_head.bbox_head.fc_reg.bias": torch.tensor(
                [
                    [3],
                    [3],
                    [3],
                    [3],
                    [4],
                    [4],
                    [4],
                    [4],
                    [5],
                    [5],
                    [5],
                    [5],
                ]
            ),
        }
        gt_dict = {
            "roi_head.bbox_head.fc_cls.weight": torch.tensor(
                [
                    [3, 3, 3, 3],
                    [2, 2, 2, 2],
                    [1, 1, 1, 1],
                ]
            ),
            "roi_head.bbox_head.fc_cls.bias": torch.tensor(
                [
                    [3],
                    [2],
                    [1],
                ]
            ),
            "roi_head.bbox_head.fc_reg.weight": torch.tensor(
                [
                    [3, 3, 3, 3],
                    [3, 3, 3, 3],
                    [3, 3, 3, 3],
                    [3, 3, 3, 3],
                    [2, 2, 2, 2],
                    [2, 2, 2, 2],
                    [2, 2, 2, 2],
                    [2, 2, 2, 2],
                    [1, 1, 1, 1],
                    [1, 1, 1, 1],
                    [1, 1, 1, 1],
                    [1, 1, 1, 1],
                ]
            ),
            "roi_head.bbox_head.fc_reg.bias": torch.tensor(
                [
                    [3],
                    [3],
                    [3],
                    [3],
                    [2],
                    [2],
                    [2],
                    [2],
                    [1],
                    [1],
                    [1],
                    [1],
                ]
            ),
        }

        class Model:
            def state_dict(self):
                return model_dict

        CustomTwoStageDetector.load_state_dict_pre_hook(Model(), model_classes, chkpt_classes, chkpt_dict, "")
        for k, gt in gt_dict.items():
            assert (chkpt_dict[k] != gt).sum() == 0



"""Unit test for otx.algorithms.common.adapters.mmcv.hooks.model_ema_hook."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import math

import torch
from mmcv.runner import BaseRunner
from mmcv.runner.hooks.ema import EMAHook

from otx.algorithms.common.adapters.mmcv.hooks import (
    CustomModelEMAHook,
    DualModelEMAHook,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockModel(torch.nn.Module):
    def __init__(self):
        super().__init__()
        self.model = torch.nn.Sequential(
            torch.nn.Conv2d(3, 8, 3),
            torch.nn.Conv2d(8, 16, 3),
            torch.nn.Conv2d(16, 64, 3),
        )

    def forward(self, x):
        return self.model(x)


class MockRunner(BaseRunner):
    class _DualModel:
        model_s = MockModel()
        model_t = MockModel()

    def __init__(self):
        self.model = self._DualModel()
        self._epoch = 0
        self._iter = 0
        self.data_loader = range(1000)

    def run(self):
        pass

    def save_checkpoint(self):
        pass

    def train(self):
        pass

    def val(self):
        pass


class TestDualModelEMAHook:
    """Test class for DualModelEMAHook."""

    @e2e_pytest_unit
    def test_before_run(self) -> None:
        """Test before_run function."""

        hook = DualModelEMAHook()
        runner = MockRunner()
        hook.before_run(runner)

    @e2e_pytest_unit
    def test_before_train_epoch(self) -> None:
        """Test before_train_epoch function."""

        hook = DualModelEMAHook(epoch_momentum=0.99)
        runner = MockRunner()
        hook.before_train_epoch(runner)
        assert hook.momentum == 1 - math.pow((1 - 0.99), (1 / 1000))
        assert hook.epoch_momentum == 0.0

    @e2e_pytest_unit
    def test_after_train_iter(self) -> None:
        """Test after_train_iter function."""

        hook = DualModelEMAHook()
        runner = MockRunner()
        # Not enable
        hook.after_train_iter(runner)

        hook.enabled = True
        hook.interval = 5

        runner._iter = 9
        # Skip
        hook.after_train_iter(runner)

        runner._iter = 0
        hook.before_run(runner)
        runner._iter = 10
        # Just copy
        hook.after_train_iter(runner)

        runner._iter = 0
        hook.before_run(runner)
        runner._epoch = 9
        # EMA
        hook.after_train_iter(runner)

    @e2e_pytest_unit
    def test_after_train_epoch(self):
        """Test after_train_epoch function."""

        hook = DualModelEMAHook()
        runner = MockRunner()
        hook.before_run(runner)
        hook.after_train_epoch(runner)


class TestCustomModelEMAHook:
    @e2e_pytest_unit
    def test_before_train_epoch(self, mocker):
        """Test before_train_epoch function."""

        mocker.patch.object(EMAHook, "before_train_epoch", return_value=True)
        hook = CustomModelEMAHook(epoch_momentum=0.99)
        runner = MockRunner()
        hook.before_train_epoch(runner)
        assert hook.momentum == 1 - math.pow((1 - 0.99), (1 / 1000))
        assert hook.epoch_momentum == 0.0



"""Task of OTX Video Recognition using mmaction training backend."""

# Copyright (C) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import glob
import os
import time
from copy import deepcopy
from functools import partial
from typing import Optional, Union

import torch
from mmaction import __version__
from mmaction.apis import train_model
from mmaction.datasets import build_dataloader, build_dataset
from mmaction.models import build_model as build_videomodel
from mmaction.utils import collect_env
from mmcv.runner import CheckpointLoader, load_state_dict, wrap_fp16_model
from mmcv.utils import Config, ConfigDict, ProgressBar, get_git_hash

from otx.algorithms.action.adapters.mmaction import (
    Exporter,
)
from otx.algorithms.action.task import OTXActionTask
from otx.algorithms.common.adapters.mmcv.utils import (
    adapt_batch_size,
    build_data_parallel,
    get_configs_by_pairs,
    patch_adaptive_interval_training,
    patch_data_pipeline,
    patch_early_stopping,
    patch_from_hyperparams,
    patch_persistent_workers,
)
from otx.algorithms.common.adapters.mmcv.utils.config_utils import (
    MPAConfig,
    update_or_add_custom_hook,
)
from otx.algorithms.common.utils import set_random_seed
from otx.algorithms.common.utils.data import get_dataset
from otx.algorithms.common.utils.logger import get_logger
from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.inference_parameters import InferenceParameters
from otx.api.entities.model import ModelPrecision
from otx.api.entities.model_template import TaskType
from otx.api.entities.subset import Subset
from otx.api.entities.task_environment import TaskEnvironment
from otx.core.data import caching

logger = get_logger()

# TODO Remove unnecessary pylint disable
# pylint: disable=too-many-lines


class MMActionTask(OTXActionTask):
    """Task class for OTX action using mmaction training backend."""

    # pylint: disable=too-many-instance-attributes
    def __init__(self, task_environment: TaskEnvironment, output_path: Optional[str] = None):
        super().__init__(task_environment, output_path)
        self._data_cfg: Optional[Config] = None
        self._recipe_cfg: Optional[Config] = None

    # pylint: disable=too-many-locals, too-many-branches, too-many-statements
    def _init_task(self, export: bool = False):  # noqa
        """Initialize task."""

        self._recipe_cfg = MPAConfig.fromfile(os.path.join(self._model_dir, "model.py"))
        self._recipe_cfg.domain = self._task_type.domain
        self._config = self._recipe_cfg

        set_random_seed(self._recipe_cfg.get("seed", 5), logger, self._recipe_cfg.get("deterministic", False))

        # Belows may go to the configure function
        patch_data_pipeline(self._recipe_cfg, self.data_pipeline_path)

        if not export:
            patch_from_hyperparams(self._recipe_cfg, self._hyperparams)
            self._recipe_cfg.total_epochs = self._recipe_cfg.runner.max_epochs

        if "custom_hooks" in self.override_configs:
            override_custom_hooks = self.override_configs.pop("custom_hooks")
            for override_custom_hook in override_custom_hooks:
                update_or_add_custom_hook(self._recipe_cfg, ConfigDict(override_custom_hook))
        if len(self.override_configs) > 0:
            logger.info(f"before override configs merging = {self._recipe_cfg}")
            self._recipe_cfg.merge_from_dict(self.override_configs)
            logger.info(f"after override configs merging = {self._recipe_cfg}")

        # add Cancel training hook
        update_or_add_custom_hook(
            self._recipe_cfg,
            ConfigDict(type="CancelInterfaceHook", init_callback=self.on_hook_initialized),
        )
        if self._time_monitor is not None:
            update_or_add_custom_hook(
                self._recipe_cfg,
                ConfigDict(
                    type="OTXProgressHook",
                    time_monitor=self._time_monitor,
                    verbose=True,
                    priority=71,
                ),
            )
        self._recipe_cfg.log_config.hooks.append({"type": "OTXLoggerHook", "curves": self._learning_curves})

        # Update recipe with caching modules
        self._update_caching_modules(self._recipe_cfg.data)

        logger.info("initialized.")

    def build_model(
        self,
        cfg: Config,
        fp16: bool = False,
        **kwargs,
    ) -> torch.nn.Module:
        """Build model from model_builder."""
        model_builder = getattr(self, "model_builder", build_videomodel)
        model = model_builder(cfg.model, **kwargs)
        ckpt = CheckpointLoader.load_checkpoint(cfg.load_from, map_location="cpu")
        if "model" in ckpt:
            ckpt = ckpt["model"]
        if "state_dict" in ckpt:
            ckpt = ckpt["state_dict"]
        load_state_dict(model, ckpt)
        if fp16:
            wrap_fp16_model(model)
        return model

    # pylint: disable=too-many-arguments
    def configure(
        self,
        training=True,
        subset="train",
        ir_options=None,
    ):
        """Patch mmcv configs for OTX action settings."""

        # deepcopy all configs to make sure
        # changes under MPA and below does not take an effect to OTX for clear distinction
        recipe_cfg = deepcopy(self._recipe_cfg)
        data_cfg = deepcopy(self._data_cfg)
        assert recipe_cfg is not None, "'recipe_cfg' is not initialized."

        recipe_cfg.work_dir = self._output_path
        recipe_cfg.resume = self._resume
        recipe_cfg.distributed = False
        recipe_cfg.omnisource = False

        if data_cfg is not None:
            recipe_cfg.merge_from_dict(data_cfg)

        if self._task_type == TaskType.ACTION_CLASSIFICATION:
            _dataset_type = "OTXActionClsDataset"
        else:
            _dataset_type = "OTXActionDetDataset"
        for subset in ("train", "val", "test", "unlabeled"):
            _cfg = recipe_cfg.data.get(subset, None)
            if not _cfg:
                continue
            _cfg.type = _dataset_type
            while "dataset" in _cfg:
                _cfg = _cfg.dataset
            _cfg.labels = self._labels

        if self._task_type == TaskType.ACTION_CLASSIFICATION:
            recipe_cfg.model["cls_head"].num_classes = len(self._labels)
        elif self._task_type == TaskType.ACTION_DETECTION:
            recipe_cfg.model["roi_head"]["bbox_head"].num_classes = len(self._labels) + 1
            if len(self._labels) < 5:
                recipe_cfg.model["roi_head"]["bbox_head"]["topk"] = len(self._labels) - 1

        recipe_cfg.data.videos_per_gpu = recipe_cfg.data.pop("samples_per_gpu", None)

        patch_adaptive_interval_training(recipe_cfg)
        patch_early_stopping(recipe_cfg)
        patch_persistent_workers(recipe_cfg)

        if self._model_ckpt is not None:
            recipe_cfg.load_from = self._model_ckpt

        self._config = recipe_cfg
        return recipe_cfg

    # pylint: disable=too-many-branches, too-many-statements
    def _train_model(
        self,
        dataset: DatasetEntity,
    ):
        """Train function in MMActionTask."""
        logger.info("init data cfg.")
        self._data_cfg = ConfigDict(data=ConfigDict())

        for cfg_key, subset in zip(
            ["train", "val", "unlabeled"],
            [Subset.TRAINING, Subset.VALIDATION, Subset.UNLABELED],
        ):
            subset = get_dataset(dataset, subset)
            if subset and self._data_cfg is not None:
                self._data_cfg.data[cfg_key] = ConfigDict(
                    otx_dataset=subset,
                    labels=self._labels,
                )

        self._is_training = True

        self._init_task()

        cfg = self.configure(True, "train", None)
        logger.info("train!")

        timestamp = time.strftime("%Y%m%d_%H%M%S", time.localtime())

        # Environment
        logger.info(f"cfg.gpu_ids = {cfg.gpu_ids}, distributed = {cfg.distributed}")
        env_info_dict = collect_env()
        env_info = "\n".join([(f"{k}: {v}") for k, v in env_info_dict.items()])
        dash_line = "-" * 60 + "\n"
        logger.info(f"Environment info:\n{dash_line}{env_info}\n{dash_line}")

        # Data
        datasets = [build_dataset(cfg.data.train)]

        # FIXME: Currently action do not support multi batch evaluation. This will be fixed
        if "val" in cfg.data:
            cfg.data.val_dataloader["videos_per_gpu"] = 1

        # Target classes
        if "task_adapt" in cfg:
            target_classes = cfg.task_adapt.get("final", [])
        else:
            target_classes = datasets[0].CLASSES

        # Metadata
        meta = dict()
        meta["env_info"] = env_info
        meta["seed"] = cfg.seed
        meta["exp_name"] = cfg.work_dir
        if cfg.checkpoint_config is not None:
            cfg.checkpoint_config.meta = dict(
                mmaction2_version=__version__ + get_git_hash()[:7],
                CLASSES=target_classes,
            )

        # Model
        model = self.build_model(cfg, fp16=cfg.get("fp16", False))
        model.train()
        model.CLASSES = target_classes

        if cfg.distributed:
            torch.nn.SyncBatchNorm.convert_sync_batchnorm(model)

        validate = bool(cfg.data.get("val", None))

        if self._hyperparams.learning_parameters.auto_decrease_batch_size:
            train_func = partial(train_model, meta=deepcopy(meta), model=deepcopy(model), distributed=False)
            adapt_batch_size(train_func, cfg, datasets, validate)

        train_model(
            model,
            datasets,
            cfg,
            distributed=cfg.distributed,
            validate=validate,
            timestamp=timestamp,
            meta=meta,
        )

        # Save outputs
        output_ckpt_path = os.path.join(cfg.work_dir, "latest.pth")
        best_ckpt_path = glob.glob(os.path.join(cfg.work_dir, "best_*.pth"))
        if best_ckpt_path:
            output_ckpt_path = best_ckpt_path[0]
        return dict(
            final_ckpt=output_ckpt_path,
        )

    def _infer_model(
        self,
        dataset: DatasetEntity,
        inference_parameters: Optional[InferenceParameters] = None,
    ):
        """Main infer function."""
        self._data_cfg = ConfigDict(
            data=ConfigDict(
                train=ConfigDict(
                    otx_dataset=None,
                    labels=self._labels,
                ),
                test=ConfigDict(
                    otx_dataset=dataset,
                    labels=self._labels,
                ),
            )
        )

        dump_features = False
        dump_saliency_map = False

        self._init_task()

        cfg = self.configure(False, "test", None)
        logger.info("infer!")

        videos_per_gpu = cfg.data.test_dataloader.get("videos_per_gpu", 1)

        # Data loader
        mm_dataset = build_dataset(cfg.data.test)
        dataloader = build_dataloader(
            mm_dataset,
            videos_per_gpu=videos_per_gpu,
            workers_per_gpu=cfg.data.test_dataloader.get("workers_per_gpu", 0),
            num_gpus=len(cfg.gpu_ids),
            dist=cfg.distributed,
            seed=cfg.get("seed", None),
            shuffle=False,
        )

        # Target classes
        if "task_adapt" in cfg:
            target_classes = cfg.task_adapt.final
            if len(target_classes) < 1:
                raise KeyError(
                    f"target_classes={target_classes} is empty check the metadata from model ckpt or recipe "
                    "configuration"
                )
        else:
            target_classes = mm_dataset.CLASSES

        # Model
        model = self.build_model(cfg, fp16=cfg.get("fp16", False))
        model.CLASSES = target_classes
        model.eval()
        model = build_data_parallel(model, cfg, distributed=False)

        # InferenceProgressCallback (Time Monitor enable into Infer task)
        time_monitor = None
        if cfg.get("custom_hooks", None):
            time_monitor = [hook.time_monitor for hook in cfg.custom_hooks if hook.type == "OTXProgressHook"]
            time_monitor = time_monitor[0] if time_monitor else None
        if time_monitor is not None:

            # pylint: disable=unused-argument
            def pre_hook(module, inp):
                time_monitor.on_test_batch_begin(None, None)

            def hook(module, inp, outp):
                time_monitor.on_test_batch_end(None, None)

            model.register_forward_pre_hook(pre_hook)
            model.register_forward_hook(hook)

        eval_predictions = []
        feature_vectors = []
        saliency_maps = []

        def dump_features_hook():
            raise NotImplementedError("get_feature_vector function for mmaction is not implemented")

        # pylint: disable=unused-argument
        def dummy_dump_features_hook(model, inp, out):
            feature_vectors.append(None)

        def dump_saliency_hook():
            raise NotImplementedError("get_saliency_map for mmaction is not implemented")

        # pylint: disable=unused-argument
        def dummy_dump_saliency_hook(model, inp, out):
            saliency_maps.append(None)

        feature_vector_hook = dump_features_hook if dump_features else dummy_dump_features_hook
        saliency_map_hook = dump_saliency_hook if dump_saliency_map else dummy_dump_saliency_hook

        prog_bar = ProgressBar(len(dataloader))
        with model.module.backbone.register_forward_hook(feature_vector_hook):
            with model.module.backbone.register_forward_hook(saliency_map_hook):
                for data in dataloader:
                    with torch.no_grad():
                        result = model(return_loss=False, **data)
                    eval_predictions.extend(result)
                    for _ in range(len(data)):
                        prog_bar.update()
        prog_bar.file.write("\n")

        for key in ["interval", "tmpdir", "start", "gpu_collect", "save_best", "rule", "dynamic_intervals"]:
            cfg.evaluation.pop(key, None)

        metric = None
        metric_name = self._recipe_cfg.evaluation.final_metric
        if inference_parameters:
            if inference_parameters.is_evaluation:
                metric = mm_dataset.evaluate(eval_predictions, **self._recipe_cfg.evaluation)[metric_name]

        assert len(eval_predictions) == len(feature_vectors), f"{len(eval_predictions)} != {len(feature_vectors)}"
        assert len(eval_predictions) == len(saliency_maps), f"{len(eval_predictions)} != {len(saliency_maps)}"
        predictions = zip(eval_predictions, feature_vectors, saliency_maps)

        return predictions, metric

    def _export_model(self, precision: ModelPrecision, dump_features: bool = True):
        """Main export function."""
        self._init_task(export=True)

        cfg = self.configure(False, "test", None)
        deploy_cfg = self._init_deploy_cfg(cfg)

        state_dict = torch.load(self._model_ckpt)
        if "model" in state_dict.keys():
            state_dict = state_dict["model"]

        self._precision[0] = precision
        half_precision = precision == ModelPrecision.FP16

        exporter = Exporter(cfg, state_dict, deploy_cfg, f"{self._output_path}/openvino", half_precision)
        exporter.export()
        bin_file = [f for f in os.listdir(self._output_path) if f.endswith(".bin")][0]
        xml_file = [f for f in os.listdir(self._output_path) if f.endswith(".xml")][0]
        onnx_file = [f for f in os.listdir(self._output_path) if f.endswith(".onnx")][0]
        results = {
            "outputs": {
                "bin": os.path.join(self._output_path, bin_file),
                "xml": os.path.join(self._output_path, xml_file),
                "onnx": os.path.join(self._output_path, onnx_file),
            }
        }
        return results

    # This should be removed
    def update_override_configurations(self, config):
        """Update override_configs."""
        logger.info(f"update override config with: {config}")
        config = ConfigDict(**config)
        self.override_configs.update(config)

    # This should moved somewhere
    def _init_deploy_cfg(self, cfg: Config) -> Union[Config, None]:
        base_dir = os.path.abspath(os.path.dirname(self._task_environment.model_template.model_template_path))
        deploy_cfg_path = os.path.join(base_dir, "deployment.py")
        deploy_cfg = None
        if os.path.exists(deploy_cfg_path):
            deploy_cfg = MPAConfig.fromfile(deploy_cfg_path)

            def patch_input_preprocessing(deploy_cfg):
                normalize_cfg = get_configs_by_pairs(
                    cfg.data.test.pipeline,
                    dict(type="Normalize"),
                )
                assert len(normalize_cfg) == 1
                normalize_cfg = normalize_cfg[0]

                options = dict(flags=[], args={})
                # NOTE: OTX loads image in RGB format
                # so that `to_rgb=True` means a format change to BGR instead.
                # Conventionally, OpenVINO IR expects a image in BGR format
                # but OpenVINO IR under OTX assumes a image in RGB format.
                #
                # `to_rgb=True` -> a model was trained with images in BGR format
                #                  and a OpenVINO IR needs to reverse input format from RGB to BGR
                # `to_rgb=False` -> a model was trained with images in RGB format
                #                   and a OpenVINO IR does not need to do a reverse
                if normalize_cfg.get("to_rgb", False):
                    options["flags"] += ["--reverse_input_channels"]
                # value must be a list not a tuple
                if normalize_cfg.get("mean", None) is not None:
                    options["args"]["--mean_values"] = list(normalize_cfg.get("mean"))
                if normalize_cfg.get("std", None) is not None:
                    options["args"]["--scale_values"] = list(normalize_cfg.get("std"))

                # fill default
                backend_config = deploy_cfg.backend_config
                if backend_config.get("mo_options") is None:
                    backend_config.mo_options = ConfigDict()
                mo_options = backend_config.mo_options
                if mo_options.get("args") is None:
                    mo_options.args = ConfigDict()
                if mo_options.get("flags") is None:
                    mo_options.flags = []

                # already defiend options have higher priority
                options["args"].update(mo_options.args)
                mo_options.args = ConfigDict(options["args"])
                # make sure no duplicates
                mo_options.flags.extend(options["flags"])
                mo_options.flags = list(set(mo_options.flags))

            patch_input_preprocessing(deploy_cfg)
            if not deploy_cfg.backend_config.get("model_inputs", []):
                raise NotImplementedError("Video recognition task must specify model input info in deployment.py")

        return deploy_cfg

    # These need to be moved somewhere
    def _update_caching_modules(self, data_cfg: Config) -> None:
        def _find_max_num_workers(cfg: dict):
            num_workers = [0]
            for key, value in cfg.items():
                if key == "workers_per_gpu" and isinstance(value, int):
                    num_workers += [value]
                elif isinstance(value, dict):
                    num_workers += [_find_max_num_workers(value)]

            return max(num_workers)

        def _get_mem_cache_size():
            if not hasattr(self._hyperparams.algo_backend, "mem_cache_size"):
                return 0

            return self._hyperparams.algo_backend.mem_cache_size

        max_num_workers = _find_max_num_workers(data_cfg)
        mem_cache_size = _get_mem_cache_size()

        mode = "multiprocessing" if max_num_workers > 0 else "singleprocessing"
        caching.MemCacheHandlerSingleton.create(mode, mem_cache_size)

        update_or_add_custom_hook(
            self._recipe_cfg,
            ConfigDict(type="MemCacheHook", priority="VERY_LOW"),
        )



"""Initial file for mmdetection models."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from . import backbones, dense_heads, detectors, heads, losses, necks, roi_heads

__all__ = ["backbones", "dense_heads", "detectors", "heads", "losses", "necks", "roi_heads"]



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-Lggggcense-Identifier: Apache-2.0
#

import pytest
import torch

from otx.core.ov.ops.generation import RangeV4
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestRangeV4:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.shape = (4, 3, 256, 256)
        self.input = torch.randn(self.shape)

    @e2e_pytest_unit
    def test_forward(self):
        op = RangeV4("dummy", shape=self.shape, output_type="i32")
        assert torch.equal(
            op(torch.tensor(0), torch.tensor(10), torch.tensor(1)),
            torch.arange(0, 10, 1),
        )

        op = RangeV4("dummy", shape=self.shape, output_type="f32")
        assert torch.equal(
            op(torch.tensor(0), torch.tensor(10), torch.tensor(1)),
            torch.arange(0, 10, 1, dtype=torch.float32),
        )



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import datetime
from typing import List

import cv2
import numpy as np
import pytest

from otx.api.entities.annotation import (
    Annotation,
    AnnotationSceneEntity,
    AnnotationSceneKind,
)
from otx.api.entities.color import Color
from otx.api.entities.coordinate import Coordinate
from otx.api.entities.id import ID
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.scored_label import ScoredLabel
from otx.api.entities.shapes.ellipse import Ellipse
from otx.api.entities.shapes.polygon import Point, Polygon
from otx.api.entities.shapes.rectangle import Rectangle
from otx.api.utils.shape_drawer import DrawerEntity, Helpers, ShapeDrawer
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements

RANDOM_IMAGE = (np.random.randint(low=0, high=255, size=(1024, 1280, 3))).astype("uint8")


class CommonMethods:
    @staticmethod
    def labels() -> List[LabelEntity]:
        creation_date = datetime.datetime(year=2021, month=12, day=9)
        detection_label = LabelEntity(
            name="Label for Detection",
            domain=Domain.DETECTION,
            color=Color(red=100, green=200, blue=150),
            creation_date=creation_date,
            id=ID("detection_label"),
        )
        segmentation_label = LabelEntity(
            name="Label for Segmentation",
            domain=Domain.DETECTION,
            color=Color(red=50, green=80, blue=200),
            creation_date=creation_date,
            is_empty=True,
            id=ID("segmentation_label"),
        )
        return [detection_label, segmentation_label]

    @staticmethod
    def scored_labels() -> List[ScoredLabel]:
        creation_date = datetime.datetime(year=2021, month=11, day=20)
        classification_label = LabelEntity(
            name="Label for Classification",
            domain=Domain.CLASSIFICATION,
            color=Color(red=200, green=170, blue=90),
            creation_date=creation_date,
            id=ID("classification_label"),
        )
        anomaly_detection_label = LabelEntity(
            name="Label for Anomaly Detection",
            domain=Domain.ANOMALY_DETECTION,
            color=Color(red=100, green=200, blue=190),
            creation_date=creation_date,
            is_empty=True,
            id=ID("anomaly_detection_label"),
        )
        return [ScoredLabel(classification_label), ScoredLabel(anomaly_detection_label)]


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestDrawerEntity:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_drawer_entity(self):
        """
        <b>Description:</b>
        Check DrawerEntity class

        <b>Input data:</b>
        DrawerEntity object

        <b>Expected results:</b>
        Test passes if DrawerEntity object "draw" method raises "NotImplementedError" exception
        """
        entity = Rectangle.generate_full_box()
        labels = CommonMethods.scored_labels()
        with pytest.raises(NotImplementedError):
            DrawerEntity().draw(RANDOM_IMAGE, entity, labels)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestHelpers:
    @staticmethod
    def generate_expected_image_with_text(
        raw_image,
        text: str,
        initial_text_color: tuple,
        processed_text_color: tuple,
        helpers: Helpers,
        expected_width: int,
        expected_height: int,
        expected_baseline: int,
        text_scale: float,
        thickness: float,
    ) -> np.array:
        processed_image = raw_image.copy()
        helpers.draw_transparent_rectangle(
            img=processed_image,
            x1=int(helpers.cursor_pos.x),
            y1=int(helpers.cursor_pos.y),
            x2=int(helpers.cursor_pos.x + expected_width),
            y2=int(helpers.cursor_pos.y + expected_height),
            color=(
                int(initial_text_color[0]),
                int(initial_text_color[1]),
                int(initial_text_color[2]),
            ),
            alpha=helpers.alpha_labels,
        )
        processed_image = cv2.putText(
            img=processed_image,
            text=text,
            org=(
                helpers.cursor_pos.x + helpers.content_padding,
                helpers.cursor_pos.y + expected_height - helpers.content_padding - expected_baseline,
            ),
            fontFace=cv2.FONT_HERSHEY_SIMPLEX,
            fontScale=text_scale,
            color=processed_text_color,
            thickness=thickness,
            lineType=cv2.LINE_AA,
        )
        return processed_image

    def generate_image_for_labels(
        self,
        expected_image: np.array,
        labels: list,
        show_labels: bool,
        show_confidence: bool,
        helpers: Helpers,
    ) -> np.array:
        expected_width = 0
        expected_height = 0
        expected_text_scale = helpers.generate_text_scale(expected_image)
        expected_thickness = int(expected_text_scale / 2)
        for label in labels:
            label_text = helpers.generate_text_for_label(label, show_labels, show_confidence)
            label_color = label.color.bgr_tuple
            label_size = cv2.getTextSize(
                label_text,
                cv2.FONT_HERSHEY_SIMPLEX,
                fontScale=expected_text_scale,
                thickness=expected_thickness,
            )
            label_baseline = label_size[1]
            label_text_width = label_size[0][0]
            label_text_height = label_size[0][1]
            label_width = label_text_width + 2 * helpers.content_padding
            label_height = label_text_height + label_baseline + 2 * helpers.content_padding
            expected_image = self.generate_expected_image_with_text(
                expected_image,
                label_text,
                label_color,
                (255, 255, 255),  # white color
                helpers,
                label_width,
                label_height,
                label_baseline,
                expected_text_scale,
                expected_thickness,
            )
            label_content_width = label_width + helpers.content_margin
            helpers.cursor_pos.x += label_content_width
            helpers.line_height = label_height
            expected_width += label_content_width
            expected_height = label_height
        return expected_image, expected_width, expected_height

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_initialization(self):
        """
        <b>Description:</b>
        Check Helpers class object initialization

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if attributes of initialized Helpers class object are equal to expected
        """
        helpers = Helpers()
        assert helpers.alpha_shape == 100 / 256
        assert helpers.alpha_labels == 153 / 256
        assert helpers.assumed_image_width_for_text_scale == 1280
        assert helpers.top_margin == 0.07
        assert helpers.content_padding == 3
        assert helpers.top_left_box_thickness == 1
        assert helpers.content_margin == 2
        assert helpers.label_offset_box_shape == 0
        assert helpers.black == (0, 0, 0)
        assert helpers.white == (255, 255, 255)
        assert helpers.yellow == (255, 255, 0)
        assert helpers.cursor_pos == Coordinate(0, 0)
        assert helpers.line_height == 0

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_draw_transparent_rectangle(self):
        """
        <b>Description:</b>
        Check Helpers class "draw_transparent_rectangle" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if image array returned by "draw_transparent_rectangle" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw_transparent_rectangle" method for rectangle inscribed to image
        2. Check array returned by "draw_transparent_rectangle" method for rectangle equal to image shape
        3. Check array returned by "draw_transparent_rectangle" method for rectangle of out-of-bounds image
        """

        def generate_image_with_rectangle(
            raw_image: np.ndarray,
            start_x: int,
            start_y: int,
            end_x: int,
            end_y: int,
            new_color: tuple,
            new_alpha: float,
        ) -> np.ndarray:
            new_image = raw_image.copy()
            cropped_rectangle = new_image[start_y:end_y, start_x:end_x]
            cropped_rectangle[:] = (new_alpha * np.array(new_color))[np.newaxis, np.newaxis] + (
                1 - new_alpha
            ) * cropped_rectangle
            return new_image

        helpers = Helpers()
        # Checking array returned by "draw_transparent_rectangle" method for rectangle inscribed to image
        image = RANDOM_IMAGE.copy()
        x1, y1, x2, y2 = 200, 100, 1100, 700
        color = (100, 50, 200)
        alpha = 0.8
        expected_image = generate_image_with_rectangle(image, x1, y1, x2 + 1, y2 + 1, color, alpha)
        helpers.draw_transparent_rectangle(image, x1, y1, x2, y2, color, alpha)
        assert np.array_equal(image, expected_image)
        # Checking array returned by "draw_transparent_rectangle" method for rectangle equal to image shape
        image = RANDOM_IMAGE.copy()
        x1, y1, x2, y2 = 0, 0, 1280, 1024
        color = (200, 80, 160)
        alpha = 0.4
        expected_image = generate_image_with_rectangle(image, x1, y1, x2 - 1, y2 - 1, color, alpha)
        helpers.draw_transparent_rectangle(image, x1, y1, x2, y2, color, alpha)
        assert np.array_equal(image, expected_image)
        # Checking array returned by "draw_transparent_rectangle" method for rectangle of out-of-bounds image
        image = RANDOM_IMAGE.copy()
        x2, y2 = 1300, 1100
        color = (70, 90, 20)
        alpha = 0.1
        expected_image = generate_image_with_rectangle(
            image, x1, y1, image.shape[1] - 1, image.shape[0] - 1, color, alpha
        )
        helpers.draw_transparent_rectangle(image, x1, y1, x2, y2, color, alpha)
        assert np.array_equal(image, expected_image)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_generate_text_scale(self):
        """
        <b>Description:</b>
        Check Helpers class "generate_text_scale" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if value returned by "generate_text_scale" method is equal to expected

        <b>Steps</b>
        1. Check value returned by "generate_text_scale" method for image width less than
        "assumed_image_width_for_text_scale" attribute of Helpers object
        2. Check value returned by "generate_text_scale" method for image width equal to
        "assumed_image_width_for_text_scale" attribute of Helpers object
        3. Check value returned by "generate_text_scale" method for image width more than
        "assumed_image_width_for_text_scale" attribute of Helpers object
        """
        helpers = Helpers()
        # Checking value returned by "generate_text_scale" for image width less than assumed_image_width_for_text_scale
        image = np.random.uniform(low=0.0, high=255.0, size=(16, 10, 3))
        assert helpers.generate_text_scale(image) == 0
        image = np.random.uniform(low=0.0, high=255.0, size=(16, 1279, 3))
        assert helpers.generate_text_scale(image) == 1
        # Checking value returned by "generate_text_scale" for image width less than assumed_image_width_for_text_scale
        image = np.random.uniform(low=0.0, high=255.0, size=(16, 1280, 3))
        assert helpers.generate_text_scale(image) == 1
        # Checking value returned by "generate_text_scale" for image width more than assumed_image_width_for_text_scale
        image = np.random.uniform(low=0.0, high=255.0, size=(16, 1281, 3))
        assert helpers.generate_text_scale(image) == 1
        image = np.random.uniform(low=0.0, high=255.0, size=(16, 2561, 3))
        assert helpers.generate_text_scale(image) == 2

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_generate_text_for_label(self):
        """
        <b>Description:</b>
        Check Helpers class "generate_text_for_label" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if value returned by "generate_text_for_label" method is equal to expected

        <b>Steps</b>
        1. Check value returned by "generate_text_for_label" for LabelEntity-type object specified as "label" parameter
        2. Check value returned by "generate_text_for_label" for ScoredLabel-type object specified as "label" parameter
        """
        helpers = Helpers()
        # Checking value returned by "generate_text_for_label" for LabelEntity object specified as "label" parameter
        labels = CommonMethods.labels()
        assert helpers.generate_text_for_label(labels[0], True, True) == "Label for Detection"
        assert helpers.generate_text_for_label(labels[0], True, False) == "Label for Detection"
        assert helpers.generate_text_for_label(labels[1], False, True) == ""
        assert helpers.generate_text_for_label(labels[1], False, False) == ""
        # Checking value returned by "generate_text_for_label" for ScoredLabel object specified as "label" parameter
        scored_labels = CommonMethods.scored_labels()
        assert helpers.generate_text_for_label(scored_labels[0], True, True) == "Label for Classification 0%"
        assert helpers.generate_text_for_label(scored_labels[0], True, False) == "Label for Classification"
        assert helpers.generate_text_for_label(scored_labels[1], False, True) == "0%"
        assert helpers.generate_text_for_label(scored_labels[1], False, False) == ""

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_generate_draw_command_for_text(self):
        """
        <b>Description:</b>
        Check Helpers class "generate_draw_command_for_text" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if tuple returned by "generate_draw_command_for_text" method is equal to expected
        """
        text = "Text to add"
        text_scale = 1.1
        thickness = 2
        expected_label_size = cv2.getTextSize(text, cv2.FONT_HERSHEY_SIMPLEX, fontScale=text_scale, thickness=thickness)
        expected_baseline = expected_label_size[1]
        expected_text_width = expected_label_size[0][0]
        expected_text_height = expected_label_size[0][1]
        expected_width = expected_text_width + 2 * 3  # text_width + 2*padding
        expected_content_width = expected_width + 2  # expected_width + margin
        expected_height = expected_text_height + expected_baseline + 2 * 3  # text_height + baseline + 2*padding
        for text_color, expected_text_color in [
            ((150, 40, 100), (255, 255, 255)),  # black text
            ((240, 250, 255), (0, 0, 0)),  # white text
        ]:
            helpers = Helpers()
            draw_command = helpers.generate_draw_command_for_text(text, text_scale, thickness, text_color)
            assert draw_command[1] == expected_content_width
            assert draw_command[2] == expected_height
            image = RANDOM_IMAGE.copy()
            expected_image = self.generate_expected_image_with_text(
                raw_image=image,
                text=text,
                initial_text_color=text_color,
                processed_text_color=expected_text_color,
                helpers=helpers,
                expected_width=expected_width,
                expected_height=expected_height,
                expected_baseline=expected_baseline,
                text_scale=text_scale,
                thickness=thickness,
            )
            actual_image = draw_command[0](image)
            assert np.array_equal(actual_image, expected_image)
            assert helpers.cursor_pos.x == expected_content_width
            assert helpers.line_height == expected_height

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_generate_draw_command_for_label(self):
        """
        <b>Description:</b>
        Check Helpers class "generate_draw_command_for_labels" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if tuple returned by "generate_draw_command_for_labels" method is equal to expected
        """
        helpers = Helpers()
        expected_helpers = Helpers()
        image = RANDOM_IMAGE.copy()
        expected_image = image.copy()
        labels = CommonMethods.labels() + CommonMethods.scored_labels()
        for show_labels in [True, False]:
            for show_confidence in [True, False]:
                (expected_image, expected_width, expected_height,) = self.generate_image_for_labels(
                    expected_image=expected_image,
                    labels=labels,
                    show_labels=show_labels,
                    show_confidence=show_confidence,
                    helpers=expected_helpers,
                )
                draw_command = helpers.generate_draw_command_for_labels(
                    labels=labels,
                    image=image,
                    show_labels=show_labels,
                    show_confidence=show_confidence,
                )
                assert draw_command[1] == expected_width
                assert draw_command[2] == expected_height
                actual_image = draw_command[0](image)
                assert np.array_equal(actual_image, expected_image)
                assert helpers.cursor_pos.x == expected_helpers.cursor_pos.x
                assert helpers.cursor_pos.y == expected_helpers.cursor_pos.y

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_draw_flagpole(self):
        """
        <b>Description:</b>
        Check Helpers class "draw_flagpole" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if array returned by "draw_flagpole" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw_flagpole" for "coordinates" parameter that fits image borders
        2. Check array returned by "draw_flagpole" for "coordinates" parameter that matches image borders
        3. Check array returned by "draw_flagpole" for "coordinates" parameter that out of image borders
        """
        helpers = Helpers()
        # Checking array returned by "draw_flagpole" for "coordinates" parameter that fits image bounds
        image = RANDOM_IMAGE.copy()
        expected_image = image.copy()
        start_point = Coordinate(1.0, 1.0)
        end_point = Coordinate(1279, 1023)
        actual_image = helpers.draw_flagpole(image, start_point, end_point)
        expected_image = cv2.line(expected_image, (1, 1), (1279, 1023), color=[0, 0, 0], thickness=2)
        assert np.array_equal(actual_image, expected_image)
        # Checking array returned by "draw_flagpole" for "coordinates" parameter that match image borders
        start_point = Coordinate(0.0, 1024.0)
        end_point = Coordinate(1280.0, 0.0)
        actual_image = helpers.draw_flagpole(image, start_point, end_point)
        expected_image = cv2.line(expected_image, (0, 1024), (1280, 0), color=[0, 0, 0], thickness=2)
        assert np.array_equal(actual_image, expected_image)
        # Checking array returned by "draw_flagpole" for "coordinates" parameter that out of image borders
        start_point = Coordinate(0.0, 0.0)
        end_point = Coordinate(1281, 1025)
        actual_image = helpers.draw_flagpole(image, start_point, end_point)
        expected_image = cv2.line(expected_image, (0, 0), (1281, 1025), color=[0, 0, 0], thickness=2)
        assert np.array_equal(actual_image, expected_image)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_newline(self):
        """
        <b>Description:</b>
        Check Helpers class "newline" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if "cursor_pos.x" and "cursor_pos.y" attributes of Helpers object returned after "newline" method
        are equal to expected
        """
        helpers = Helpers()
        helpers.newline()
        assert helpers.cursor_pos.x == 0  # resets to 0
        assert helpers.cursor_pos.y == 2  # pos.y + content_margin(equal to 2)
        helpers.cursor_pos.x = 10
        helpers.newline()
        assert helpers.cursor_pos.x == 0
        assert helpers.cursor_pos.y == 4

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_helpers_set_cursor_pos(self):
        """
        <b>Description:</b>
        Check Helpers class "set_cursor_pos" method

        <b>Input data:</b>
        Helpers class object

        <b>Expected results:</b>
        Test passes if "cursor_pos.x" and "cursor_pos.y" attributes of Helpers class object returned after
        "set_cursor_pos" method are equal to expected

        <b>Steps</b>
        1. Check cursor_pos.x and cursor_pos.y attributes after "set_cursor_pos" with specified "cursor_pos" parameter
        2. Check cursor_pos.x and cursor_pos.y attributes after "set_cursor_pos" with not specified "cursor_pos"
        parameter
        """
        helpers = Helpers()
        # Checking cursor_pos.x and cursor_pos.y after "set_cursor_pos" method with specified "cursor_pos" parameter
        new_position = Coordinate(60.0, 200.0)
        helpers.set_cursor_pos(new_position)
        assert helpers.cursor_pos.x is new_position.x
        assert helpers.cursor_pos.y is new_position.y
        # Checking cursor_pos.x and cursor_pos.y after "set_cursor_pos" method with non-specified "cursor_pos" parameter
        helpers.set_cursor_pos()
        assert helpers.cursor_pos.x == 0
        assert helpers.cursor_pos.y == 0


class ShapeDrawerParams:
    @staticmethod
    def full_rectangle_labels() -> List[LabelEntity]:
        rectangle_label = LabelEntity(
            name="Full-Rectangle Annotation Label",
            domain=Domain.DETECTION,
            color=Color(100, 200, 60),
            creation_date=datetime.datetime(year=2021, month=12, day=16),
            id=ID("full_rectangle_label_1"),
        )
        other_rectangle_label = LabelEntity(
            name="other Full-Rectangle Annotation Label",
            domain=Domain.SEGMENTATION,
            color=Color(80, 160, 200),
            creation_date=datetime.datetime(year=2021, month=12, day=15),
            id=ID("full_rectangle_label_2"),
        )
        return [rectangle_label, other_rectangle_label]

    def full_rectangle_scored_labels(self) -> List[ScoredLabel]:
        labels = self.full_rectangle_labels()
        return [ScoredLabel(labels[0]), ScoredLabel(labels[1])]

    def full_rectangle_annotation(self) -> Annotation:
        return Annotation(
            shape=Rectangle(x1=0, y1=0, x2=1, y2=1),
            labels=self.full_rectangle_scored_labels(),
            id=ID("full_rectangle_annotation"),
        )

    @staticmethod
    def rectangle_labels() -> List[LabelEntity]:
        rectangle_label = LabelEntity(
            name="Rectangle Annotation Label",
            domain=Domain.DETECTION,
            color=Color(100, 200, 60),
            creation_date=datetime.datetime(year=2021, month=12, day=16),
            id=ID("rectangle_label_1"),
        )
        other_rectangle_label = LabelEntity(
            name="other Rectangle Annotation Label",
            domain=Domain.SEGMENTATION,
            color=Color(80, 160, 200),
            creation_date=datetime.datetime(year=2021, month=12, day=15),
            id=ID("rectangle_label_2"),
        )
        return [rectangle_label, other_rectangle_label]

    def rectangle_scored_labels(self) -> List[ScoredLabel]:
        labels = self.rectangle_labels()
        return [ScoredLabel(labels[0]), ScoredLabel(labels[1])]

    def rectangle_annotation(self) -> Annotation:
        return Annotation(
            shape=Rectangle(x1=0.1, y1=0.4, x2=0.4, y2=0.9),
            labels=self.rectangle_scored_labels(),
            id=ID("rectangle_annotation"),
        )

    @staticmethod
    def polygon_labels() -> List[LabelEntity]:
        polygon_label = LabelEntity(
            name="Polygon Annotation Label",
            domain=Domain.DETECTION,
            color=Color(200, 200, 100),
            creation_date=datetime.datetime(year=2021, month=12, day=16),
            id=ID("polygon_label_1"),
        )
        other_polygon_label = LabelEntity(
            name="other Polygon Annotation Label",
            domain=Domain.SEGMENTATION,
            color=Color(100, 100, 150),
            creation_date=datetime.datetime(year=2021, month=12, day=15),
            id=ID("polygon_label_2"),
        )
        return [polygon_label, other_polygon_label]

    def polygon_scored_labels(self) -> List[ScoredLabel]:
        labels = self.polygon_labels()
        return [ScoredLabel(labels[0]), ScoredLabel(labels[1])]

    def polygon_annotation(self) -> Annotation:
        return Annotation(
            shape=Polygon(
                [
                    Point(0.3, 0.4),
                    Point(0.3, 0.7),
                    Point(0.5, 0.75),
                    Point(0.8, 0.7),
                    Point(0.8, 0.4),
                ]
            ),
            labels=self.polygon_scored_labels(),
            id=ID("polygon_annotation"),
        )

    @staticmethod
    def ellipse_labels() -> List[LabelEntity]:
        ellipse_label = LabelEntity(
            name="Ellipse Annotation Label",
            domain=Domain.DETECTION,
            color=Color(100, 100, 200),
            creation_date=datetime.datetime(year=2021, month=12, day=16),
            id=ID("ellipse_label_1"),
        )
        other_ellipse_label = LabelEntity(
            name="other Ellipse Annotation Label",
            domain=Domain.SEGMENTATION,
            color=Color(200, 80, 150),
            creation_date=datetime.datetime(year=2021, month=12, day=15),
            id=ID("ellipse_label_2"),
        )
        return [ellipse_label, other_ellipse_label]

    def ellipse_scored_labels(self) -> List[ScoredLabel]:
        labels = self.ellipse_labels()
        return [ScoredLabel(labels[0]), ScoredLabel(labels[1])]

    def ellipse_annotation(self) -> Annotation:
        return Annotation(
            shape=Ellipse(x1=0.5, y1=0.0, x2=1.0, y2=0.5),
            labels=self.ellipse_scored_labels(),
            id=ID("ellipse_annotation"),
        )

    def annotation_scene(self) -> AnnotationSceneEntity:
        return AnnotationSceneEntity(
            annotations=[
                self.full_rectangle_annotation(),
                self.rectangle_annotation(),
                self.polygon_annotation(),
                self.ellipse_annotation(),
            ],
            kind=AnnotationSceneKind.ANNOTATION,
            creation_date=datetime.datetime(year=2021, month=12, day=16),
            id=ID("annotation_scene"),
        )


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestShapeDrawer:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_shape_drawer_initialization(self):
        """
        <b>Description:</b>
        Check ShapeDrawer class object initialization

        <b>Input data:</b>
        ShapeDrawer class object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized ShapeDrawer class object are equal to expected

        <b>Steps</b>
        1. Check attributes of ShapeDrawer object initialized with "show_count" parameter is "False" and "is_one_label"
        is "True"
        2. Check attributes of ShapeDrawer object initialized with "show_count" parameter is "True" and "is_one_label"
        is "True"
        3. Check attributes of ShapeDrawer object initialized with "show_count" parameter is "False" and "is_one_label"
        is "False"
        """
        for show_count, is_one_label, show_labels in [
            (False, True, False),
            (True, True, True),
            (False, False, True),
        ]:
            shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            assert shape_drawer.show_labels == show_labels
            assert shape_drawer.show_confidence
            assert shape_drawer.show_count == show_count
            assert shape_drawer.is_one_label == is_one_label
            for drawer in shape_drawer.shape_drawers:
                assert drawer.show_labels == show_labels
                assert drawer.show_confidence
            assert shape_drawer.top_left_drawer.show_labels
            assert shape_drawer.top_left_drawer.show_confidence
            assert shape_drawer.top_left_drawer.is_one_label == is_one_label

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_shape_drawer_draw(self):
        """
        <b>Description:</b>
        Check ShapeDrawer class "draw" method

        <b>Input data:</b>
        ShapeDrawer class object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if image array returned by "draw" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw" method for "show_count" parameter is "True" and "is_one_label" is "True
        2. Check array returned by "draw" method for "show_count" parameter is "False" and "is_one_label" is "False"
        3. Check array returned by "draw" method for "show_count" parameter is "True" and "is_one_label" is "False"
        4. Check array returned by "draw" method for "show_count" parameter is "False" and "is_one_label" is "True"
        """
        annotation_scene = ShapeDrawerParams().annotation_scene()
        annotations = annotation_scene.annotations
        full_rectangle_annotation = annotations[0]
        rectangle_annotation = annotations[1]
        polygon_annotation = annotations[2]
        ellipse_annotation = annotations[3]

        for show_count, is_one_label in [
            (True, True),
            (False, False),
            (True, False),
            (False, True),
        ]:
            image = RANDOM_IMAGE.copy()
            expected_image = image.copy()
            shape_drawer = ShapeDrawer(show_count, is_one_label)
            if not shape_drawer.is_one_label:
                expected_image = shape_drawer.top_left_drawer.draw(expected_image, full_rectangle_annotation, labels=[])
            expected_image = shape_drawer.shape_drawers[0].draw(
                expected_image,
                rectangle_annotation.shape,
                rectangle_annotation.get_labels(),
            )
            expected_image = shape_drawer.shape_drawers[1].draw(
                expected_image,
                polygon_annotation.shape,
                polygon_annotation.get_labels(),
            )
            expected_image = shape_drawer.shape_drawers[2].draw(
                expected_image,
                ellipse_annotation.shape,
                ellipse_annotation.get_labels(),
            )
            if is_one_label:
                expected_image = shape_drawer.top_left_drawer.draw_labels(expected_image, annotation_scene.get_labels())
            if show_count:
                expected_image = shape_drawer.top_left_drawer.draw_annotation_count(expected_image, 3)
            shape_drawer.top_left_drawer.set_cursor_pos()
            actual_image = shape_drawer.draw(image, annotation_scene, [])
            assert np.array_equal(actual_image, expected_image)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestTopLeftDrawer:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_top_left_drawer_initialization(self):
        """
        <b>Description:</b>
        Check TopLeftDrawer subclass object initialization

        <b>Input data:</b>
        TopLeftDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized TopLeftDrawer subclass object are equal to expected

        <b>Steps</b>
        1. Check attributes of TopLeftDrawer object initialized for ShapeDrawer with "show_count" parameter is "False"
        and "is_one_label" is "True"
        2. Check attributes of TopLeftDrawer object initialized for ShapeDrawer with "show_count" parameter is "True"
        and "is_one_label" is "True"
        3. Check attributes of TopLeftDrawer object initialized for ShapeDrawer with "show_count" parameter is "False"
        and "is_one_label" is "False"
        """
        for show_count, is_one_label in [(False, True), (True, True), (False, False)]:
            shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            assert shape_drawer.top_left_drawer.show_labels
            assert shape_drawer.top_left_drawer.show_confidence
            assert shape_drawer.top_left_drawer.is_one_label == is_one_label

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_top_left_drawer_draw_labels(self):
        """
        <b>Description:</b>
        Check TopLeftDrawer subclass "draw_labels" method

        <b>Input data:</b>
        TopLeftDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if image array returned by "draw_labels" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw_labels" method for ShapeDrawer with "show_count" parameter is "False" and
        "is_one_label" is "True"
        2. Check array returned by "draw_labels" method for ShapeDrawer with "show_count" parameter is "True" and
        "is_one_label" is "True"
        3. Check array returned by "draw_labels" method for ShapeDrawer with "show_count" parameter is "False" and
        "is_one_label" is "False"
        """
        labels = ShapeDrawerParams.rectangle_labels() + ShapeDrawerParams().polygon_scored_labels()
        for show_count, is_one_label in [(False, True), (True, True), (False, False)]:
            image = RANDOM_IMAGE.copy()
            expected_image = image.copy()
            shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            expected_shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            show_confidence = (
                shape_drawer.top_left_drawer.show_confidence if not shape_drawer.top_left_drawer.is_one_label else False
            )
            expected_image = TestHelpers().generate_image_for_labels(
                expected_image,
                labels,
                expected_shape_drawer.top_left_drawer.show_labels,
                show_confidence,
                expected_shape_drawer.top_left_drawer,
            )[0]
            expected_shape_drawer.top_left_drawer.newline()
            actual_image = shape_drawer.top_left_drawer.draw_labels(image, labels)
            assert np.array_equal(actual_image, expected_image)
            assert shape_drawer.top_left_drawer.cursor_pos == expected_shape_drawer.top_left_drawer.cursor_pos

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_top_left_drawer_draw(self):
        """
        <b>Description:</b>
        Check TopLeftDrawer subclass "draw" method

        <b>Input data:</b>
        TopLeftDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if image array returned by "draw" method is equal to expected
        """
        annotation = ShapeDrawerParams().annotation_scene()
        image = RANDOM_IMAGE.copy()
        expected_image = image.copy()
        shape_drawer = ShapeDrawer(True, True)
        expected_shape_drawer = ShapeDrawer(True, True)
        draw_command = expected_shape_drawer.top_left_drawer.generate_draw_command_for_labels(
            annotation.get_labels(), expected_image, True, True
        )[0]
        expected_image = draw_command(expected_image)
        actual_image = shape_drawer.top_left_drawer.draw(image, annotation, [])
        assert np.array_equal(actual_image, expected_image)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_top_left_drawer_draw_annotation_count(self):
        """
        <b>Description:</b>
        Check TopLeftDrawer subclass "draw_annotation_count" method

        <b>Input data:</b>
        TopLeftDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if image array returned by "draw_annotation_count" method is equal to expected
        """
        image = RANDOM_IMAGE.copy()
        expected_image = image.copy()
        shape_drawer = ShapeDrawer(True, True)
        expected_shape_drawer = ShapeDrawer(True, True)
        draw_command = expected_shape_drawer.top_left_drawer.generate_draw_command_for_text(
            "Count: 4", 1.0, 1, (255, 255, 0)
        )[0]
        expected_image = draw_command(expected_image)
        expected_shape_drawer.top_left_drawer.newline()
        actual_image = shape_drawer.top_left_drawer.draw_annotation_count(image, 4)
        assert np.array_equal(actual_image, expected_image)
        assert shape_drawer.top_left_drawer.cursor_pos == expected_shape_drawer.top_left_drawer.cursor_pos


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestRectangleDrawer:
    @staticmethod
    def draw_rectangle_labels(
        image: np.array,
        labels: list,
        shape_drawer: ShapeDrawer,
        rectangle: Rectangle,
        cursor_position: Coordinate,
    ) -> np.ndarray:

        image_copy = image.copy()
        rectangle_drawer = shape_drawer.shape_drawers[0]
        base_color = labels[0].color.bgr_tuple
        x1, y1 = int(rectangle.x1 * image_copy.shape[1]), int(rectangle.y1 * image_copy.shape[0])
        x2, y2 = int(rectangle.x2 * image_copy.shape[1]), int(rectangle.y2 * image_copy.shape[0])
        # Drawing rectangle
        image_copy = rectangle_drawer.draw_transparent_rectangle(
            image_copy, x1, y1, x2, y2, base_color, rectangle_drawer.alpha_shape
        )
        # Drawing rectangle frame
        image_copy = cv2.rectangle(img=image_copy, pt1=(x1, y1), pt2=(x2, y2), color=base_color, thickness=2)
        # Generating draw command to add labels to image
        draw_command, _, _ = rectangle_drawer.generate_draw_command_for_labels(
            labels,
            image_copy,
            rectangle_drawer.show_labels,
            rectangle_drawer.show_confidence,
        )
        rectangle_drawer.set_cursor_pos(cursor_position)
        image_copy = draw_command(image_copy)
        return image_copy

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_rectangle_drawer_initialization(self):
        """
        <b>Description:</b>
        Check RectangleDrawer subclass object initialization

        <b>Input data:</b>
        RectangleDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized RectangleDrawer subclass object are equal to expected
        """
        for show_count, is_one_label, show_labels in [
            (False, True, False),
            (True, True, True),
            (False, False, True),
        ]:
            shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            assert shape_drawer.shape_drawers[0].show_labels == show_labels
            assert shape_drawer.shape_drawers[0].show_confidence

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_rectangle_drawer_draw(self):
        """
        <b>Description:</b>
        Check RectangleDrawer subclass "draw" method

        <b>Input data:</b>
        RectangleDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if image array returned by "draw" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw" method without changing labels positions
        2. Check array returned by "draw" method with putting labels to the bottom of drawn rectangle
        3. Check array returned by "draw" method with shifting labels to the left of drawn rectangle
        """
        labels = ShapeDrawerParams().rectangle_scored_labels()
        for rectangle, expected_cursor_position in [
            (  # without changing labels positions
                Rectangle(0.1, 0.3, 0.8, 0.5),
                Coordinate(128, 271),
            ),
            (  # with putting labels to the bottom of drawn rectangle
                Rectangle(0.1, 0.1, 0.9, 0.9),
                Coordinate(128, 102),
            ),
            (  # with shifting labels to the left of drawn rectangle
                Rectangle(0.6, 0.7, 0.9, 0.9),
                Coordinate(61, 680),
            ),
        ]:
            image = RANDOM_IMAGE.copy()
            shape_drawer = ShapeDrawer(True, False)
            expected_shape_drawer = ShapeDrawer(True, False)
            expected_image = self.draw_rectangle_labels(
                image,
                labels,
                expected_shape_drawer,
                rectangle,
                expected_cursor_position,
            )
            actual_image = shape_drawer.shape_drawers[0].draw(image, rectangle, labels)
            assert np.array_equal(actual_image, expected_image)
            assert shape_drawer.top_left_drawer.cursor_pos == expected_shape_drawer.top_left_drawer.cursor_pos


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestEllipseDrawer:
    @staticmethod
    def draw_ellipse_labels(
        image: np.array,
        labels: list,
        shape_drawer: ShapeDrawer,
        ellipse: Ellipse,
        cursor_position: Coordinate,
        flagpole_start: Coordinate,
        flagpole_end: Coordinate,
    ) -> np.ndarray:
        image_copy = image.copy()
        ellipse_shape_drawer = shape_drawer.shape_drawers[2]
        base_color = labels[0].color.bgr_tuple
        if ellipse.width > ellipse.height:
            axes = (
                int(ellipse.major_axis * image_copy.shape[1]),
                int(ellipse.minor_axis * image_copy.shape[0]),
            )
        else:
            axes = (
                int(ellipse.major_axis * image_copy.shape[0]),
                int(ellipse.minor_axis * image_copy.shape[1]),
            )
        center = (
            int(ellipse.x_center * image_copy.shape[1]),
            int(ellipse.y_center * image_copy.shape[0]),
        )
        # Drawing Ellipse on image
        overlay = cv2.ellipse(
            img=image_copy.copy(),
            center=center,
            axes=axes,
            angle=0,
            startAngle=0,
            endAngle=360,
            color=base_color,
            thickness=cv2.FILLED,
        )
        result_without_border = cv2.addWeighted(
            overlay,
            ellipse_shape_drawer.alpha_shape,
            image_copy,
            1 - ellipse_shape_drawer.alpha_shape,
            0,
        )
        # Drawing Ellipse borders on image
        result_with_border = cv2.ellipse(
            img=result_without_border,
            center=center,
            axes=axes,
            angle=0,
            startAngle=0,
            endAngle=360,
            color=base_color,
            lineType=cv2.LINE_AA,
        )
        # Generating draw command to add labels to image
        draw_command = ellipse_shape_drawer.generate_draw_command_for_labels(
            labels,
            image_copy,
            ellipse_shape_drawer.show_labels,
            ellipse_shape_drawer.show_confidence,
        )[0]
        # Getting top-left corner of box around Ellipse
        ellipse_shape_drawer.set_cursor_pos(cursor_position)
        image_copy = draw_command(result_with_border)
        image_copy = ellipse_shape_drawer.draw_flagpole(image_copy, flagpole_start, flagpole_end)
        return image_copy

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_drawer_initialization(self):
        """
        <b>Description:</b>
        Check EllipseDrawer subclass object initialization

        <b>Input data:</b>
        EllipseDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized EllipseDrawer subclass object are equal to expected
        """
        for show_count, is_one_label, show_labels in [
            (False, True, False),
            (True, True, True),
            (False, False, True),
        ]:
            shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            assert shape_drawer.shape_drawers[2].show_labels == show_labels
            assert shape_drawer.shape_drawers[2].show_confidence

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_drawer_draw(self):
        """
        <b>Description:</b>
        Check EllipseDrawer subclass "draw" method

        <b>Input data:</b>
        EllipseDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if array returned by "draw" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw" method without changing labels positions
        2. Check array returned by "draw" method with putting labels to the bottom
        3. Check array returned by "draw" method with shifting labels to the left
        """
        labels = ShapeDrawerParams().ellipse_scored_labels()
        for (ellipse, expected_cursor_position, flagpole_start, flagpole_end,) in [
            (  # without changing labels positions
                Ellipse(0.1, 0.3, 0.8, 0.5),
                Coordinate(128.0, 271.2),
                Coordinate(129.0, 307.2),
                Coordinate(129, 409),
            ),
            (  # with putting labels to the bottom
                Ellipse(0.1, 0.1, 0.8, 0.8),
                Coordinate(128.0, 921.6),
                Coordinate(129.0, 921.6),
                Coordinate(129, 460),
            ),
            (  # with shifting labels to the left
                Ellipse(0.6, 0.7, 0.9, 0.9),
                Coordinate(299, 680.8),
                Coordinate(769.0, 716.8),
                Coordinate(769, 819),
            ),
        ]:
            image = RANDOM_IMAGE.copy()
            shape_drawer = ShapeDrawer(True, False)
            expected_shape_drawer = ShapeDrawer(True, False)
            expected_image = self.draw_ellipse_labels(
                image,
                labels,
                expected_shape_drawer,
                ellipse,
                expected_cursor_position,
                flagpole_start,
                flagpole_end,
            )
            actual_image = shape_drawer.shape_drawers[2].draw(image, ellipse, labels)
            assert np.array_equal(actual_image, expected_image)
            assert shape_drawer.top_left_drawer.cursor_pos == expected_shape_drawer.top_left_drawer.cursor_pos


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestPolygonDrawer:
    @staticmethod
    def draw_polygon_labels(
        image: np.array,
        labels: list,
        shape_drawer: ShapeDrawer,
        polygon: Polygon,
        cursor_position: Coordinate,
        flagpole_start: Coordinate,
        flagpole_end: Coordinate,
    ) -> np.ndarray:
        image_copy = image.copy()
        polygon_drawer = shape_drawer.shape_drawers[1]
        base_color = labels[0].color.bgr_tuple
        # Draw Polygon on the image
        alpha = polygon_drawer.alpha_shape
        contours = np.array(
            [[point.x * image_copy.shape[1], point.y * image_copy.shape[0]] for point in polygon.points],
            dtype=np.int32,
        )
        overlay = cv2.drawContours(
            image=image_copy.copy(),
            contours=[contours],
            contourIdx=-1,
            color=base_color,
            thickness=cv2.FILLED,
        )
        result_without_border = cv2.addWeighted(overlay, alpha, image_copy, 1 - alpha, 0)
        result_with_border = cv2.drawContours(
            image=result_without_border,
            contours=[contours],
            contourIdx=-1,
            color=base_color,
            thickness=2,
            lineType=cv2.LINE_AA,
        )
        # Generating draw command to add labels to image
        draw_command = polygon_drawer.generate_draw_command_for_labels(
            labels,
            image_copy,
            polygon_drawer.show_labels,
            polygon_drawer.show_confidence,
        )[0]
        # Getting top-left corner of box around Polygon
        polygon_drawer.set_cursor_pos(cursor_position)
        image_copy = draw_command(result_with_border)
        image_copy = polygon_drawer.draw_flagpole(image_copy, flagpole_start, flagpole_end)
        return image_copy

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon_drawer_initialization(self):
        """
        <b>Description:</b>
        Check PolygonDrawer subclass object initialization

        <b>Input data:</b>
        PolygonDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized PolygonDrawer subclass object are equal to expected
        """
        for show_count, is_one_label, show_labels in [
            (False, True, False),
            (True, True, True),
            (False, False, True),
        ]:
            shape_drawer = ShapeDrawer(show_count=show_count, is_one_label=is_one_label)
            assert shape_drawer.shape_drawers[1].show_labels == show_labels
            assert shape_drawer.shape_drawers[1].show_confidence

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon_drawer_draw(self):
        """
        <b>Description:</b>
        Check PolygonDrawer subclass "draw" method

        <b>Input data:</b>
        PolygonDrawer subclass object with "show_count" and "is_one_label" parameters

        <b>Expected results:</b>
        Test passes if image array returned by "draw" method is equal to expected

        <b>Steps</b>
        1. Check array returned by "draw" method without changing labels positions
        2. Check array returned by "draw" method with putting labels to the bottom
        3. Check array returned by "draw" method with shifting labels to the left
        """
        labels = ShapeDrawerParams().polygon_scored_labels()
        polygon_no_change_labels_position = Polygon(
            [
                Point(0.2, 0.2),
                Point(0.2, 0.6),
                Point(0.4, 0.7),
                Point(0.6, 0.6),
                Point(0.6, 0.2),
            ]
        )
        polygon_put_labels_to_bottom = Polygon(
            [
                Point(0.2, 0.1),
                Point(0.2, 0.6),
                Point(0.4, 0.7),
                Point(0.6, 0.6),
                Point(0.6, 0.1),
            ]
        )
        polygon_shift_labels_to_left = Polygon(
            [
                Point(0.4, 0.2),
                Point(0.4, 0.5),
                Point(0.6, 0.6),
                Point(0.9, 0.5),
                Point(0.9, 0.2),
            ]
        )

        for (polygon, expected_cursor_position, flagpole_start, flagpole_end,) in [
            (  # without changing labels position
                polygon_no_change_labels_position,
                Coordinate(251, 168),
                Coordinate(257, 204),
                Coordinate(257, 204),
            ),
            (  # with putting labels to the bottom
                polygon_put_labels_to_bottom,
                Coordinate(251, 716),
                Coordinate(257, 716),
                Coordinate(257, 102),
            ),
            (  # with shifting labels to the left
                polygon_shift_labels_to_left,
                Coordinate(251, 168),
                Coordinate(513, 204),
                Coordinate(513, 204),
            ),
        ]:
            image = RANDOM_IMAGE.copy()
            shape_drawer = ShapeDrawer(True, False)
            expected_shape_drawer = ShapeDrawer(True, False)
            expected_image = self.draw_polygon_labels(
                image,
                labels,
                expected_shape_drawer,
                polygon,
                expected_cursor_position,
                flagpole_start,
                flagpole_end,
            )
            actual_image = shape_drawer.shape_drawers[1].draw(image, polygon, labels)
            assert np.array_equal(actual_image, expected_image)
            assert shape_drawer.top_left_drawer.cursor_pos == expected_shape_drawer.top_left_drawer.cursor_pos



import atheris


class FuzzingHelper(object):
    """Helper to make required data from input_bytes for the fuzzing tests"""

    def __init__(self, input_bytes):
        """Init"""
        self.provider = atheris.FuzzedDataProvider(input_bytes)

    def get_string(self, byte_conut=256):
        """Consume a string"""
        return self.provider.ConsumeString(byte_conut)



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest

from otx.api.configuration.enums.model_lifecycle import ModelLifecycle
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestModelLifecycle:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_model_lifecycle(self):
        """
        <b>Description:</b>
        Check ModelLifecycle Enum class elements

        <b>Expected results:</b>
        Test passes if ModelLifecycle Enum class length is equal to expected value and its elements have expected
        sequence numbers and values returned by __str__ method
        """
        assert len(ModelLifecycle) == 5
        assert ModelLifecycle.NONE.value == 1
        assert str(ModelLifecycle.NONE) == "NONE"
        assert ModelLifecycle.ARCHITECTURE.value == 2
        assert str(ModelLifecycle.ARCHITECTURE) == "ARCHITECTURE"
        assert ModelLifecycle.TRAINING.value == 3
        assert str(ModelLifecycle.TRAINING) == "TRAINING"
        assert ModelLifecycle.INFERENCE.value == 4
        assert str(ModelLifecycle.INFERENCE) == "INFERENCE"
        assert ModelLifecycle.TESTING.value == 5
        assert str(ModelLifecycle.TESTING) == "TESTING"



# Copyright (C) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

from pathlib import Path

import pytest
import yaml

from otx.api.configuration import ConfigurableParameters, cfg_helper
from otx.api.entities.id import ID
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.label_schema import LabelSchemaEntity
from otx.api.entities.model_template import parse_model_template
from otx.api.entities.task_environment import TaskEnvironment
from tests.test_helpers import ConfigExample
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


def __get_path_to_file(filename: str):
    """
    Return the path to the file named 'filename', which lives in the tests/entities directory
    """
    return str(Path(__file__).parent / Path(filename))


def dummy_config():
    """
    Return dict from yaml file
    """
    cur_config = __get_path_to_file("./dummy_config.yaml")
    with open(cur_config, "r") as stream:
        return yaml.safe_load(stream)


def environment():
    """
    Return TaskEnvironment
    """
    car = LabelEntity(id=ID(123456789), name="car", domain=Domain.DETECTION, is_empty=False)
    person = LabelEntity(id=ID(987654321), name="person", domain=Domain.DETECTION, is_empty=False)
    labels_list = [car, person]
    dummy_template = __get_path_to_file("./dummy_template.yaml")
    model_template = parse_model_template(dummy_template)
    hyper_parameters = model_template.hyper_parameters.data
    params = cfg_helper.create(hyper_parameters)
    labels_schema = LabelSchemaEntity.from_labels(labels_list)
    environment = TaskEnvironment(
        model=None,
        hyper_parameters=params,
        label_schema=labels_schema,
        model_template=model_template,
    )
    return environment


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestTaskEnvironment:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_task_environment(self):
        """
        <b>Description:</b>
        Check the TaskEnvironment can correctly return the value

        <b>Input data:</b>
        Dummy data

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Using an already created dummy environment.
        2. Checking class fields
        """

        env = environment()
        __dummy_config = dummy_config()

        assert env == TaskEnvironment(
            model=None,
            model_template=env.model_template,
            hyper_parameters=env.get_hyper_parameters(),
            label_schema=env.label_schema,
        )
        assert isinstance(env, TaskEnvironment)
        assert env != "Fail params"
        assert len(env.get_labels()) == 2

        for i in ["header", "description", "visible_in_ui"]:
            assert getattr(env.get_model_configuration().configurable_parameters, i) == __dummy_config[i]

        assert env.get_model_configuration().configurable_parameters.id == ID()

        for param in __dummy_config:
            getattr(env.get_hyper_parameters(), param) == __dummy_config[param]

        assert env.get_hyper_parameters().id == ID()

        assert "model=None" in repr(env)
        assert "label_schema=LabelSchemaEntity(label_groups=[LabelGroup(id=" in repr(env)
        assert "name=from_label_list" in repr(env)
        assert "group_type=LabelGroupType.EXCLUSIVE" in repr(env)
        assert "labels=[LabelEntity(123456789, name=car, hotkey=, domain=DETECTION" in repr(env)
        assert "LabelEntity(987654321, name=person, hotkey=, domain=DETECTION" in repr(env)
        assert "CONFIGURABLE_PARAMETERS(header='Configuration for an object detection task -- TEST ONLY'" in repr(env)
        assert "description='Configuration for an object detection task -- TEST ONLY'" in repr(env)
        assert "visible_in_ui=True" in repr(env)
        assert "id=ID()" in repr(env)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_learning_parameters_num_workers(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml num_workers default_value

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking num_workers default_value from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_learning_parameters_num_workers = (
            env.get_hyper_parameters().learning_parameters.num_workers
        )  # "default_value"
        config_example_learning_parameters_num_workers = (
            config_example.learning_parameters._default.factory.num_workers.metadata["default_value"]
        )

        # From dummy_config.yaml because it is missing in dummy_template.yaml "parameter_overrides"
        assert env_learning_parameters_num_workers == 0
        assert config_example_learning_parameters_num_workers == 4  # From ConfigExample

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_learning_parameters_batch_size(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml batch_size default_value

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking batch_size from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_learning_parameters_batch_size = (
            env.get_hyper_parameters().learning_parameters.batch_size
        )  # "default_value"
        config_example_learning_parameters_batch_size = (
            config_example.learning_parameters._default.factory.batch_size.metadata["default_value"]
        )

        assert env_learning_parameters_batch_size == 64  # From dummy_template.yaml "parameter_overrides"
        assert config_example_learning_parameters_batch_size == 5  # From ConfigExample

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_learning_parameters_num_iters(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml num_iters default_value

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking num_iters from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_learning_parameters_num_iters = env.get_hyper_parameters().learning_parameters.num_iters  # "default_value"
        config_example_learning_parameters_num_iters = (
            config_example.learning_parameters._default.factory.num_iters.metadata["default_value"]
        )

        assert env_learning_parameters_num_iters == 13000  # From dummy_template.yaml "parameter_overrides"
        assert config_example_learning_parameters_num_iters == 1  # From ConfigExample

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_learning_parameters_learning_rate(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml learning_rate default_value

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking learning_rate from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_learning_parameters_learning_rate = (
            env.get_hyper_parameters().learning_parameters.learning_rate
        )  # "default_value"
        config_example_learning_parameters_learning_rate = (
            config_example.learning_parameters._default.factory.learning_rate.metadata["default_value"]
        )

        assert env_learning_parameters_learning_rate == 0.05  # From dummy_template.yaml "parameter_overrides"
        assert config_example_learning_parameters_learning_rate == 0.01  # From ConfigExample

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_learning_parameters_num_checkpoints(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml num_checkpoints default_value
        "num_checkpoints" is missing in ConfigExample

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking num_checkpoints from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_learning_parameters_num_checkpoints = (
            env.get_hyper_parameters().learning_parameters.num_checkpoints
        )  # "default_value"

        # From dummy_config.yaml because it is missing in dummy_template.yaml "parameter_overrides"
        assert env_learning_parameters_num_checkpoints == 5

        # Attempt to access the missing parameter in ConfigExample
        with pytest.raises(AttributeError):
            # AttributeError: type object '__LearningParameters' has no attribute 'num_checkpoints'
            config_example.learning_parameters._default.factory.num_checkpoints.metadata["default_value"]

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_dummy_config_missing_param(self):
        """
        <b>Description:</b>
        Check dummy_config missing_param

        <b>Input data:</b>
        dummy_config

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking missing_param dummy_config
        """

        env = environment()

        # Attempt to access the missing parameter in dummy_config.yaml
        with pytest.raises(AttributeError):
            # AttributeError: 'PARAMETER_GROUP' object has no attribute 'missing_param'
            env.get_hyper_parameters().learning_parameters.missing_param

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_postprocessing_confidence_threshold(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml confidence_threshold default_value

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking confidence_threshold from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_postprocessing_confidence_threshold = env.get_hyper_parameters().postprocessing.confidence_threshold
        cep = config_example.postprocessing
        config_example_postprocessing_confidence_threshold = cep._default.factory.confidence_threshold.metadata[
            "default_value"
        ]

        # From dummy_config.yaml because it is missing in dummy_template.yaml "parameter_overrides"
        assert env_postprocessing_confidence_threshold == 0.35
        assert config_example_postprocessing_confidence_threshold == 0.25  # From ConfigExample

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_compare_postprocessing_result_based_confidence_threshold(self):
        """
        <b>Description:</b>
        Check matches ConfigExample and dummy_config.yaml result_based_confidence_threshold default_value

        <b>Input data:</b>
        ConfigExample, dummy_config.yaml

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking result_based_confidence_threshold from ConfigExample and dummy_config.yaml
        """

        env = environment()
        config_example = env.get_hyper_parameters(ConfigExample)

        env_postprocessing_result_based_confidence_threshold = (
            env.get_hyper_parameters().postprocessing.result_based_confidence_threshold
        )  # "default_value"
        cep = config_example.postprocessing
        def_factory = cep._default.factory
        rbct = def_factory.result_based_confidence_threshold
        config_example_postprocessing_result_based_confidence_threshold = rbct.metadata["default_value"]

        # From dummy_config.yaml because it is missing in dummy_template.yaml "parameter_overrides"
        assert env_postprocessing_result_based_confidence_threshold is True
        assert config_example_postprocessing_result_based_confidence_threshold is True  # From ConfigExample

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_set_hyper_parameters(self):
        """
        <b>Description:</b>
        Check set_hyper_parameters() method

        <b>Input data:</b>
        Dummmy data

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Checking parameters after setting
        """
        env = environment()

        header = "Test header"
        description = "Test description"
        visible_in_ui = False
        id = ID(123456789)

        hyper_parameters = ConfigurableParameters(
            header=header, description=description, visible_in_ui=visible_in_ui, id=id
        )
        env.set_hyper_parameters(hyper_parameters=hyper_parameters)
        assert env.get_hyper_parameters().header == header
        assert env.get_hyper_parameters().description == description
        assert env.get_hyper_parameters().visible_in_ui == visible_in_ui
        assert env.get_hyper_parameters().id == id

        assert env.get_model_configuration().configurable_parameters.header == header
        assert env.get_model_configuration().configurable_parameters.description == description
        assert env.get_model_configuration().configurable_parameters.visible_in_ui == visible_in_ui
        assert env.get_model_configuration().configurable_parameters.id == id

        with pytest.raises(ValueError):
            # ValueError: Unable to set hyper parameters, invalid input: 123
            env.set_hyper_parameters(hyper_parameters="123")



"""MMseg model builder."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from copy import deepcopy
from typing import Optional, Union

import torch
from mmcv.runner import load_checkpoint
from mmcv.utils import Config, ConfigDict
from mmseg.models.builder import MODELS

SCALAR_SCHEDULERS = MODELS


def build_scalar_scheduler(cfg, default_value=None):
    """Build scalar scheduler."""
    if cfg is None:
        if default_value is not None:
            assert isinstance(default_value, (int, float))
            cfg = dict(type="ConstantScalarScheduler", scale=float(default_value))
        else:
            return None
    elif isinstance(cfg, (int, float)):
        cfg = dict(type="ConstantScalarScheduler", scale=float(cfg))

    return SCALAR_SCHEDULERS.build(cfg)


def build_segmentor(
    config: Config,
    train_cfg: Optional[Union[Config, ConfigDict]] = None,
    test_cfg: Optional[Union[Config, ConfigDict]] = None,
    checkpoint: Optional[str] = None,
    device: Union[str, torch.device] = "cpu",
    cfg_options: Optional[Union[Config, ConfigDict]] = None,
    from_scratch: bool = False,
    is_training: bool = False,
) -> torch.nn.Module:
    """A builder function for mmseg model.

    Creates a model, based on the configuration in config.
    Note that this function updates 'load_from' attribute of 'config'.
    """

    # fmt: off
    # isort: off
    # false positive (mypy, pylint)
    # pylint: disable-next=no-name-in-module
    from mmseg.models import build_segmentor as origin_build_segmentor  # type: ignore[attr-defined]
    # isort: on
    # fmt: on

    if cfg_options is not None:
        config.merge_from_dict(cfg_options)

    model_cfg = deepcopy(config.model)
    model = origin_build_segmentor(model_cfg, train_cfg=train_cfg, test_cfg=test_cfg)
    model = model.to(device)

    checkpoint = checkpoint if checkpoint else config.get("load_from", None)
    config.load_from = checkpoint

    if checkpoint is not None and not from_scratch:
        load_checkpoint(model, checkpoint, map_location=device)
        if is_training is True:
            config.load_from = None  # To prevent the repeated ckpt loading in mmseg.apis.train_segmentor

    return model



"""Composed dataloader."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from otx.algorithms.common.utils.logger import get_logger

logger = get_logger()


class CDLIterator:
    """Iterator for aligning the number of batches as many as samples in the first iterator."""

    def __init__(self, cdl):
        self._cdl = cdl
        self._index = 0
        self._cdl_iter = [iter(dl) for dl in self._cdl.loaders]

    def __next__(self):
        """Generate the next batch."""
        if self._index < self._cdl.max_iter:
            batches = {}
            for i, iterator in enumerate(self._cdl_iter):
                if i == 0:
                    batches = next(iterator)
                else:
                    try:
                        batches[f"extra_{i-1}"] = next(iterator)
                    except StopIteration:
                        self._cdl_iter[1] = iter(self._cdl.loaders[1])
                        batches[f"extra_{i-1}"] = next(self._cdl_iter[1])
            self._index += 1
            return batches
        raise StopIteration


class ComposedDL:
    """Composed dataloader for combining two or more loaders together."""

    class DummySampler:
        """Dummy sampler class to relay set_epoch() call to the list of data loaders in the CDL."""

        def __init__(self, cdl):
            self.cdl = cdl

        def set_epoch(self, epoch):
            """Set epoch."""
            loaders = self.cdl.loaders
            for loader in loaders:
                loader.sampler.set_epoch(epoch)

    def __init__(self, loaders=None):
        if loaders is None:
            loaders = []
        self.loaders = loaders
        self.max_iter = len(self.loaders[0])
        logger.info(f"possible max iterations = {self.max_iter}")
        self._sampler = ComposedDL.DummySampler(self)

    def __len__(self):
        """Return length of the first loader."""
        return self.max_iter

    def __iter__(self):
        """Iter."""
        return CDLIterator(self)

    @property
    def sampler(self):
        """Return sampler."""
        return self._sampler
